/* comm -- compare two sorted files line by line. */
/*Modified for zenpakt by Anshul Narula <sunny_delhi18@yahoo.cm>*/

/*original copyright
   Copyright (C) 1986, 1990-1991, 1995-2005, 2008-2010 Free Software
   Foundation, Inc.

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */

/* Written by Richard Stallman and David MacKenzie. */


#include <config.h>
#include <locale.h>
#include <sys/types.h>
#include <unistd.h>
#include "linebuffer.h"
#include "hard-locale.h"
#include "stdio--.h"
#include "builtins.h"
#include "shell.h"
#include "bashgetopt.h"

/* Undefine, to avoid warning about redefinition on some systems.  */
#undef min
#define min(x, y) ((x) < (y) ? (x) : (y))

/* True if the LC_COLLATE locale is hard.  */
static bool hard_LC_COLLATE;

/* If true, print lines that are found in both files. */
/* If false , print lines omly in file1 */
static bool both;

/* If nonzero, we have seen at least one unpairable line. */
static bool seen_unpairable;

/* If nonzero, we have warned about disorder in that file. */
static bool issued_disorder_warning[2];

/* Output the line in linebuffer LINE to stream STREAM
   provided the switches say it should be output.
   CLASS is 1 for a line found only in file 1,
   2 for a line only in file 2, 3 for a line in both. */

static void
writeline (struct linebuffer const *line, FILE *stream, int class)
{
  switch (class)
    {
    case 1:
      if (both)
        return;
      break;

    case 3:
      if (!both)
        return;
      break;
    }

  fwrite (line->buffer, sizeof (char), line->length, stream);
}

/* Compare INFILES[0] and INFILES[1].
   If either is "-", use the standard input for that file.
   Assume that each input file is sorted;
   merge them and output the result.  */

static int
compare_files (char **infiles)
{
  /* For each file, we have four linebuffers in lba. */
  struct linebuffer lba[2][4];

  /* thisline[i] points to the linebuffer holding the next available line
     in file i, or is NULL if there are no lines left in that file.  */
  struct linebuffer *thisline[2];

  /* all_line[i][alt[i][0]] also points to the linebuffer holding the
     current line in file i. We keep two buffers of history around so we
     can look two lines back when we get to the end of a file. */
  struct linebuffer *all_line[2][4];

  /* This is used to rotate through the buffers for each input file. */
  int alt[2][3];

  /* streams[i] holds the input stream for file i.  */
  FILE *streams[2];

  int i, j;

  /* Initialize the storage. */
  for (i = 0; i < 2; i++)
    {
      for (j = 0; j < 4; j++)
        {
          initbuffer (&lba[i][j]);
          all_line[i][j] = &lba[i][j];
        }
      alt[i][0] = 0;
      alt[i][1] = 0;
      alt[i][2] = 0;
      streams[i] = (STREQ (infiles[i], "-") ? stdin : fopen (infiles[i], "r"));
      if (!streams[i])
        return (EXECUTION_FAILURE);

      thisline[i] = readlinebuffer (all_line[i][alt[i][0]], streams[i]);
      if (ferror (streams[i]))
        return (EXECUTION_FAILURE);
    }

  while (thisline[0] || thisline[1])
    {
      int order;
      bool fill_up[2] = { false, false };

      /* Compare the next available lines of the two files.  */

      if (!thisline[0])
        order = 1;
      else if (!thisline[1])
        order = -1;
      else {
        size_t len = min (thisline[0]->length, thisline[1]->length) - 1;
        order = memcmp (thisline[0]->buffer, thisline[1]->buffer, len);
        if (order == 0)
          order = (thisline[0]->length < thisline[1]->length
                   ? -1
                   : thisline[0]->length != thisline[1]->length);
       }

      /* Output the line that is lesser. */
      if (order == 0)
        writeline (thisline[1], stdout, 3);
      else
        {
          seen_unpairable = true;
          if (order <= 0)
            writeline (thisline[0], stdout, 1);
        }

      /* Step the file the line came from.
         If the files match, step both files.  */
      if (0 <= order)
        fill_up[1] = true;
      if (order <= 0)
        fill_up[0] = true;

      for (i = 0; i < 2; i++)
        if (fill_up[i])
          {
            /* Rotate the buffers for this file. */
            alt[i][2] = alt[i][1];
            alt[i][1] = alt[i][0];
            alt[i][0] = (alt[i][0] + 1) & 0x03;

            thisline[i] = readlinebuffer (all_line[i][alt[i][0]], streams[i]);

            if (ferror (streams[i]))
              return  (EXECUTION_FAILURE);

            fill_up[i] = false;
          }
    }

  for (i = 0; i < 2; i++)
    if (fclose (streams[i]) != 0)
      return (EXECUTION_FAILURE);

  return (EXECUTION_SUCCESS);
}

/*
int
comm0_main (int argc, char **argv)
{
  int opt, r, i;
   extern char *optarg;
   extern int optind, opterr, optopt;

  optind = 1;
  hard_LC_COLLATE = hard_locale (LC_COLLATE);
  atexit (close_stdout);

  both = true;

  seen_unpairable = false;
  issued_disorder_warning[0] = issued_disorder_warning[1] = false;

  while ((opt = getopt(argc, argv, "+fh")) != -1) {
     	switch (opt) {
  	case 'f':
      both = false;
  		break;
  	case 'h':
      optind = 1;
  		builtin_usage();
  		return(EXECUTION_SUCCESS);
  	default:
      optind = 1;
      builtin_usage();
  		return(EX_USAGE);
  	}
  }

  if (argc - optind != 2) {
      builtin_usage ();
      optind = 1;
      return(EX_USAGE);
    }

  i = optind;
  optind = 1;
  r = compare_files (argv + i);
  return r;
}
*/

int
comm0_builtin (list)
  WORD_LIST *list;
{

  int opt, r, c, ind = 1;
  char **v;

  v = make_builtin_argv(list, &c);

  hard_LC_COLLATE = hard_locale (LC_COLLATE);

  /*atexit (close_stdout);*/

  both = true;

  seen_unpairable = false;
  issued_disorder_warning[0] = issued_disorder_warning[1] = false;
  reset_internal_getopt ();
  while ((opt = internal_getopt (list, "fh")) != -1) {
  	switch (opt) {
  	case 'f':
      ind++;
      both = false;
  		break;
  	case 'h':
  		builtin_usage();
  		return(EXECUTION_SUCCESS);
  	default:
      builtin_usage();
  		return(EX_USAGE);
  	}
  }

  list = loptend;

  if (list_length(list) != 2) {
      builtin_usage ();
      return(EX_USAGE);
    }

  r = compare_files (v + ind);
  free (v);
/**/
/*
  v = make_builtin_argv(list, &c);
  r = comm0_main(c, v);
*/
  return (r);
}


/* An array of strings forming the `long' documentation for a builtin xxx,
   which is printed by `help xxx'.  It must end with a NULL.  By convention,
   the first line is a short description. */
char *comm0_doc[] = {
	"Compare sorted null delimited files FILE1 and FILE2",
	"With no options, print common null delimited lines in FILE1 and FILE2.",
  "With -f option, print null delimited lines unique to FILE1",
	(char *)NULL
};

/* The standard structure describing a builtin command.  bash keeps an array
   of these structures.  The flags must include BUILTIN_ENABLED so the
   builtin can be used. */
struct builtin comm0_struct = {
	"comm0",		/* builtin name */
	comm0_builtin,		/* function implementing the builtin */
	BUILTIN_ENABLED,	/* initial flags for builtin */
	comm0_doc,		/* array of long documentation strings. */
	"comm0  [ -f ] FILE1 FILE2",		/* usage synopsis; becomes short_doc */
	0			/* reserved for internal use */
};
