/*
 *      This file is part of zenpakt
 *
 *      Copyright 2011 Anshul Narula aka sunny <sunny_delhi18@yahoo.com>
 *
 *      Zenpakt is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 2 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>

#if defined (HAVE_UNISTD_H)
#  include <unistd.h>
#endif

#include <stdio.h>

#include "builtins.h"
#include "shell.h"
#include "bashgetopt.h"
#include "chop.h"
#include "escape.h"

int convesc_builtin (WORD_LIST *list)
{
  int c, null=0;
  char **v;
  ssize_t read;

  if (no_options(list)) {
    builtin_usage();
    return EX_USAGE;
  }

  v=make_builtin_argv(list,&c);

  if (c == 1) {
    char *line = NULL;
    size_t len;
    while ((read = getline(&line, &len, stdin)) != -1 ) {
      char eline[read+1];
      do_chop(line);         /**************/
      unescape(line, eline); /**************/
      printf("%s%c", eline,'\0');
    }

    if (line)
      free(line);

  } else if (c == 3) {
    if (legal_identifier (v[1]) || valid_array_reference (v[1])) {
      char eline[strlen(v[2])+1];
      do_chop(v[2]);
      unescape(v[2],eline);
      if (valid_array_reference (v[1]) == 0)
        bind_variable (v[1], eline, 0);
      else
        assign_array_element (v[1], eline, 0);
      stupidly_hack_special_variables (v[1]);
    } else {
      sh_invalidid (v[1]);
      return EX_USAGE;
    }
  } else {
    builtin_usage();
    return EX_USAGE;
  }

  return EXECUTION_SUCCESS;
}

char *convesc_doc[] = {
	"convesc [variable string]",
	"",
	"read from stdin and output null delimited unescaped strings for each line.",
	"If arguments are provided, store the unescaped result for STRING in VARIABLE",
	(char *)NULL
};

struct builtin convesc_struct = {
	"convesc",		/* builtin name */
	convesc_builtin,		/* function implementing the builtin */
	BUILTIN_ENABLED,	/* initial flags for builtin */
	convesc_doc,		/* array of long documentation strings. */
	"metaget filename",		/* usage synopsis; becomes short_doc */
	0			/* reserved for internal use */
};
