/*
 *      doinstextractlinks.c
 *
 *      Copyright 2011 Anshul Narula aka sunny <sunny_delhi18@yahoo.com>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 2 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <regex.h>

#if defined (HAVE_UNISTD_H)
#  include <unistd.h>
#endif

#include "builtins.h"
#include "shell.h"
#include "bashgetopt.h"
#include "chop.h"
#include "escape.h"

#define IS_SPACE(s) (s == ' ' || s == '\t')
#define SKIP_WHITESPACE(s) while (ISSPACE(*s)) { ++s; }

/*the state of quoting for an argument, used by return_argument*/
enum {
  STATE_NO_QUOTE,
  STATE_D_QUOTE,
  STATE_S_QUOTE,
};

/*the two modes of output*/
enum {
  DOINST_XLINKS_ESC,
  DOINST_XLINKS_UNESC
};

static char * return_argument(char **s) {

  int state=STATE_NO_QUOTE;
  char *ret, *r, *p;
  unsigned int c;
  int run=1;

  size_t size;
  p= *s;
  size=strlen(p);
  r = ret= (char *) malloc(size+1);


  for( ; *p != '\0' && run; p++) {
    c=*p;

    switch(c) {

      case '\'':
        if (state == STATE_S_QUOTE) state=STATE_NO_QUOTE;
	else if (state == STATE_D_QUOTE)
	  *ret++='\'';
	else
	  state=STATE_S_QUOTE;
	break;

      case '\"':
        if (state == STATE_D_QUOTE)
	  state=STATE_NO_QUOTE;
	else if (state == STATE_S_QUOTE)
	  *ret++='\"';
	else
	  state=STATE_D_QUOTE;
	break;

      case '\\':
        if (state == STATE_S_QUOTE)
	  *ret++='\\';
	else {
          p++; c=*p;
          if (c == '\'' && state == STATE_S_QUOTE)
	    p--;
	  else
	    *ret++=c;
	}
	break;

      case ' ':
      case '\t':
        if (state == STATE_NO_QUOTE) {
	  p--;
	  run=0;
	} else
	  *ret++=c;
	break;

      default:
        *ret++=c;
	break;
    }
  }

  *ret='\0';
  *s=p;
  return r;
}

static int extract_link(char *line, char **link_dir, char **link_name) {
  /*set to 1 when called the first time*/
  static state=0;
  char *s;
  unsigned int c;
  char *dir;
  size_t size;

  s=line;

  SKIP_WHITESPACE(s)

  if (*s++ != '(') return 0;

  if (!IS_SPACE(*s)) return 0;
  SKIP_WHITESPACE(s)

  if (*s++ != 'c' || *s++ != 'd')
    return 0;

  if (!IS_SPACE(*s)) return 0;
  SKIP_WHITESPACE(s)

  dir = *link_dir = return_argument(&s);

  if (!IS_SPACE(*s)) return 0;
  SKIP_WHITESPACE(s)

  if (*s++ != ';')
    return 0;

  if (!IS_SPACE(*s)) return 0;
  SKIP_WHITESPACE(s)

  if (*s++ != 'r' || *s++ != 'm')
    return 0;

  if (!IS_SPACE(*s)) return 0;
  SKIP_WHITESPACE(s)

  if (*s++ != '-' || *s++ != 'r' || *s++ != 'f')
    return 0;

  if (!IS_SPACE(*s)) return 0;
  SKIP_WHITESPACE(s)

  *link_name = return_argument(&s);

  if (!IS_SPACE(*s)) return 0;
  SKIP_WHITESPACE(s)

  if (*s != ')')
    return 0;

  return 1;

}

static int doinst_extract_links(char *doinst, int mode) {
  FILE * fp;
  char * line = NULL;
  size_t len = 0;
  ssize_t read;
  char *link_dir;
  char *link_name;
  char *esc_link_dir;
  char *esc_link_name;

  if (doinst) {
    fp = fopen(doinst, "r");
    if (fp == NULL)
      return 1;

    while ((read = getline(&line, &len, fp)) != -1) {
      link_dir= (char *) malloc(read+1);
      link_name= (char *) malloc(read+1);
      do_chop(line);
      if (extract_link(line,&link_dir,&link_name)) {
	if (mode == DOINST_XLINKS_ESC) {
	  esc_link_dir = escape(link_dir, NULL);
	  esc_link_name = escape(link_name, NULL);
          printf("%s/%s\n", link_dir, link_name);
	} else
          printf("%s/%s%c", link_dir, link_name,'\0');
      }
    }

  } else {

    while ((read = getline(&line, &len, stdin)) != -1) {
      link_dir= (char *) malloc(read+1);
      link_name= (char *) malloc(read+1);
      do_chop(line);
      if (extract_link(line,&link_dir,&link_name)) {
	if (mode == DOINST_XLINKS_ESC) {
	  esc_link_dir = escape(link_dir, NULL);
	  esc_link_name = escape(link_name, NULL);
          printf("%s/%s\n", link_dir, link_name);
	} else
          printf("%s/%s%c", link_dir, link_name,'\0');
      }
    }
  }

  if (link_dir)
    free(link_dir);

  if (link_name)
    free(link_name);

  if (line)
    free(line);

  return 0;
}

int doinstxlinks_builtin (WORD_LIST *list) {
  int c, i, opt, mode=DOINST_XLINKS_ESC;
  char **v;

  reset_internal_getopt ();
  while ((opt = internal_getopt (list, "n")) != -1) {
    switch (opt) {
      case 'n':
        mode=DOINST_XLINKS_UNESC;
	break;
      default:
	builtin_usage();
	return(EX_USAGE);
    }
  }

  list = loptend;
  v=make_builtin_argv(list,&c);

  if (c == 1) {
    if (doinst_extract_links(NULL, mode))
      return EXECUTION_FAILURE;
  } else {

    for(i=1 ; i < c ; i++) {
      if (doinst_extract_links(v[i], mode))
	return EXECUTION_FAILURE;
    }

  }

  return EXECUTION_SUCCESS;
}

char *doinstxlinks_doc[] = {
  "metaget [-fF] filename...",
  "",
  "get newline delimited file lists from given meta database",
  "-F specifies that print a null delimited unescaped file list.",
  "-f specifies that print a newline delimited file list without",
  "unescaping backslashed quoted characters.It is recommended you use",
  "-f as default behaviour may change in future",
  (char *)NULL
};

struct builtin doinstxlinks_struct = {
  "doinstxlinks",		/* builtin name */
  doinstxlinks_builtin,		/* function implementing the builtin */
  BUILTIN_ENABLED,	/* initial flags for builtin */
  doinstxlinks_doc,		/* array of long documentation strings. */
  "metaget [-fF] filename...",		/* usage synopsis; becomes short_doc */
  0			/* reserved for internal use */
};
