/*
 *      This file is part of zenpakt
 *
 *      Copyright 2011 Anshul Narula aka sunny <sunny_delhi18@yahoo.com>
 *
 *      Zenpakt is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 2 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include "escape.h"

/*
 *modified bash's printf.def tescape: translate the given string starting at \ in
 *estart to the escape character and return the number of
 *characters consumed to, store the character in cp
*/
int esc_to_char(char *estart, char *cp)
{
  register char *p;
  int temp, c, evalue;

  p = estart;

  switch (c = *p++)
    {
#if defined (__STDC__)
      case 'a': *cp = '\a'; break;
#else
      case 'a': *cp = '\007'; break;
#endif

      case 'b': *cp = '\b'; break;

/*    case 'e':
      case 'E': *cp = '\033'; break;	*//* ESC -- non-ANSI */

      case 'f': *cp = '\f'; break;

      case 'n': *cp = '\n'; break;

      case 'r': *cp = '\r'; break;

      case 't': *cp = '\t'; break;

      case 'v': *cp = '\v'; break;

      /*we consider \ followed by three octal digits as escape sequences*/
      case '0': case '1': case '2': case '3':
      case '4': case '5': case '6': case '7':
        evalue = OCTVALUE(c);
        for (temp = 2; ISOCTAL (*p) && temp--; p++)
	        evalue = (evalue * 8) + OCTVALUE (*p);
	      *cp = evalue & 0xFF;
	      break;

      case '\\':
       	*cp = c;
      	break;

      default:
        /*This should not happen, if \ is followed by an unrecognized character
	       *it means tar produced an unexpected output, or someone messed with the
	       *meta file list; any ways, we just ignore it and continue with our error
	       *message */
         *cp = '\\';
	       FILE * stderr=fdopen(dup(2),"w");
         fprintf(stderr, "Error: unknown escape sequence: \\%c\n", c);
				 fclose(stderr);
		     return 0;
    }
  return (p - estart);
}

/*produce an unescaped version of s and store it in t*/
int unescape(char *s, char *t)
{
int n_esc;
char cp;
char *p;

p = s;
for(; *p != '\0' ; p++, t++) {

	switch(*p) {
		/*if the character is backslash we need to get the charcter the sequence
		 *represents*/
		case '\\':

		  if (n_esc=esc_to_char(p+1, &cp)) {
				while(n_esc-- > 0)
				 	p++;
			  *t= cp;
			}
			else /*add error checking here for unknown escape  sequences*/
			  *t= cp;
      break;

		default:
		  *t=*p;
		  break;
	}
}

*t='\0';
return 0;
}

/*return escaped version of s.If len is not NULL, return the size occupied by
 *return string*/
char * escape (char *s, int *len)
{
  char *p, *r, *ret;
  int i, size;
  unsigned char c;

  if (s == NULL || *s == '\0')
    return (char *) NULL ;

  i =  strlen(s);
  size = 4 * i + 1;
  r = ret = (char *)xmalloc (size);

  for (p = s ; *s != '\0' ; s++) {
    c = *s;

    switch (c) {
#ifdef __STDC__
      case '\a':
#else
      case '\007':
#endif
	*r++ = '\\';
	*r++ = '0';
	*r++ = '0';
	*r++ = '7';
	break;

#ifdef __STDC__
      case '\v':
#else
      case 0x0b:
#endif
        *r++ = '\\';
        *r++ = '0';
        *r++ = '1';
        *r++ = '3';
	break;

      case '\033':
        *r++ = '\\';
	*r++ = '0';
	*r++ = '3';
	*r++ = '3';
	break;


      case '\b':
        *r++ = '\\';
        *r++ = 'b';
	break;

      case '\f':
        *r++ = '\\';
        *r++ = 'f';
	break;

      case '\n':
        *r++ = '\\';
        *r++ = 'n';
	break;

      case '\r':
        *r++ = '\\';
	*r++ = 'r';

      case '\t':
	*r++ = '\\';
	*r++ = 't';
	break;

      case '\\':
        *r++ = '\\';
        *r++ = '\\';
	break;

      default:
	if (ISPRINT (c) == 0) {
	  *r++ = '\\';
	  *r++ = TOCHAR ((c >> 6) & 07);
	  *r++ = TOCHAR ((c >> 3) & 07);
	  *r++ = TOCHAR (c & 07);
	} else
	  *r++ = c;
	break;
    }
  }

  *r = '\0';
  if (len)
    *len = r - ret;

  return ret;
}
