#       Copyright 2010 Anshul Narula aka sunny <sunny_delhi18@yahoo.com>
#
#       This file is part of zenpakt

#       Zenpakt is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation; either version 2 of the License, or
#       (at your option) any later version.
#
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#       MA 02110-1301, USA.

$DOC
========
 choose
========

**Select is deprecated use choose**

SYNOPSIS
--------

choose [-P error_message] [-C] [-V var] num list.. -p prompt [-ers]
       [-d delim] [-i text] [-n nchars] [-N nchars]  [-t timeout] [-u fd]

yesno msg

POSIXLY_CORRECT:
  choose [-P error_message] [-V var] num list... -p prompt [-r] [-u fd]

DESCRIPTION
-----------

calls read in a loop until one of the valid answers are provided;
This can be used in place of select builtin as it takes less space.
All the options to read can be passed.

Though possible, you must not use -a and should not specify a variable for read
for choose to work properly (instead use -V var).Also, you may specify -p prompt
to override the default prompt for selection.

_Num_ specifies number of items in the list after which options to read start.

Extra options are:
 P   error message to be displayed if wrong choice is entered
 V   store the responses in given variable
 C   case insensitive matching for valid reponses

In POSIXLY_CORRECT mode, -d, -a, -e, -s, -i, -n , -N, -t, -C are ignored and have
no effect

yesno calls choose with y yes n no as valid answers and "msg ([y]es/[n]o)? " as
prompt. If the answer is y or yes(case insensitive) returns 0 or else returns
non-zero.

$ENDDOC

$$

choose() {

  local OPTIND=1 OPTARG opt error_message list_c inarray=( inarray ) \
    var=REPLY choices || return 1

  #parse the options
  while getopts :P:CV: opt; do
    case $opt in
      P)
        error_message=$OPTARG
        ;;
      C)
        inarray+=("-PC")
        ;;
      V)
        var=$OPTARG
        ;;
      \?)
        die "$FUNCNAME:unknown option -$OPTARG"
        ;;
      :)
        die "$FUNCNAME:option -$OPTARG requires an argument"
        ;;
    esac
  done
  shift $((OPTIND-1))

  unset "$var" || return 1
  [[ $1 = *[^0-9]* ]]  && return 1
  list_c=$1

  printf -v choices "'%s'," "${@:2:list_c}"
  choices=${choices%,}

  #now we can read and verify if it is one of correct responses or not
  #please note if EOF is encountered nothing is verified
  #to chnage the behaviour change && in following line to a simple ;
  while IFS= read -p "<enter a choice ($choices)> " "${@:list_c+2}" "$var"; \
    ! "${inarray[@]}" "${!var}" "${@:2:list_c}" >/dev/null; do

    printf_v '\r%s(valid responses are: %s )\n' "${error_message:-invalid reply}" \
      "$choices" >&2

  done
}

yesno() {
  local REPLY
  choose -C 4 y yes n no -p "$1 ([y]es/[n]o)? "
  [[ $REPLY = @(y|Y)@(|[Ee][Ss]) ]]
}

Select() {
  print "zenpakt:***Select is deprecated, use choose***"
  choose "$@"
}
