#       error.bash
#       This file is part of zenpakt

#       Copyright 2011 Anshul Narula aka sunny <sunny_delhi18@yahoo.com>
#
#       Zenpakt is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation; either version 2 of the License, or
#       (at your option) any later version.
#
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#       MA 02110-1301, USA.

$$

#initial exit status is zero
#Dont use this variable directly
_error_exit_status_=0

#the error list to be printed is empty in the beggining
_error_list_=()

#the callback function called upon a call to error_continue
#in case error_continue calls continue
_error_continue_callback=

#The UID is not 0
ERR_NOTROOT=2
#error processing options
ERR_OPT=3
#no tar found
ERR_NO_TAR=4
#locking failed
ERR_LOCKED=5
#database initialization failed
ERR_DBINIT=6
#temp file/dir creation failure
ERR_CREAT_TMP=7
#no database when expected
ERR_NO_DB=8
#loading builtin failed
ERR_BUILTIN_LOAD=9
#stdin is not a tty
ERR_NO_INTTY=10
#stdout is not a tty
ERR_NO_OUTTTY=11
#stderr is not a tty
ERR_NO_ERRTTY=12
#compression utility is not present
ERR_NO_COMPRESSUTIL=13

error=(
  #this message is generally not used from array , in case sourcing this file
  #causes the error, but it is here for reference and some other uses
  [1]="failed to source functions"
  [2]="you are not root"
  [3]="invalid option/usage"
  [4]="tar not found in PATH"
  [5]="database locked by another process"
  [6]="failed to initialize package databse"
  [7]="failed to create temporary directory/file"
  [8]="package database doesnot exist"
  [9]="failed to load builtin"
  [10]="interactive mode needs standard input connected to the terminal"
  [11]="interactive mode needs standard output connected to the terminal"
  [12]="interactive mode needs standard error connected to the terminal"
  [13]="external compression utility '%s' missing"
)

#return the last set exit status
error_exit_status() {
  printf -v "$1" '%s' "$_error_exit_status_"
}

#error_geterrno num|identifier
error_geterrno() {
  if [[ $1 != *[^0-9]* && ${error[$1]} ]] ; then
    printf -v "$2" '%s' "$1"
  elif [[ $1 != _ccallback && ${error["$1"]} = *[^0-9]* && ${error[${error["$1"]}]} ]] ; then
    printf -v "$2" '%s' "${error[${error["$1"]}]}"
  else
    return 1
  fi
}

#print the error message and set the exit status
#error errno argument
error() {
  local errno continue=1

  if error_geterrno "$1" errno ; then
    _error_exit_status_=$errno
    error_print_v "${error[$errno]}"
  elif [[ $1 = +([0-9]) ]] ; then
    _error_exit_status_=$1
  else
    #invalid error identifier
    return 1
  fi
}

#exit with given exit status or last set exit status
error_exit() {
  local errno
  if [[ $1 ]] && error_geterrno "$1" errno ; then
    error_print "${error[$errno]}"
    _error_exit_status_=$errno
    exit $errno
  elif [[ $1 = +([0-9]) ]] ; then
    _error_exit_status_=$1
    exit "$1"
  else
    exit "$_error_exit_status_"
  fi
}

#$1 errno|identifier argument [continue_no]
error_continue() {
  local errno continue=1

  if error_geterrno "$1" errno ; then
    _error_exit_status_=$errno
    _error_list_+=( "$errno" "$2" )
    error_print_v "${error[$errno]}"
  elif [[ $1 = +([0-9]) ]] ; then
    _error_exit_status_=$1

  else
    #invalid error identifier
    return 1

  fi

  [[ $3 = [1-9]+([0-9]) ]] && continue=$3

  if [[ $_error_continue_callback ]] ; then
    eval "$_error_continue_callback"
  fi

  continue "$continue"

}

error_summary() {
  ((${#_error_list_[@]})) || return 0
  error_print "${1:-errors were encountered while processing following package(s)}:"
  local i
  for ((i=0 ; i < ${#_error_list_[@]} ; i+=2)) ; do
    printf '%s: %s ( %s )\n' >&2 "${_error_list_[i+1]}" "${_error_list_[i]}" \
      "${error[${_error_list_[i]}]}"
  done
}

error_print() {
  spin_inactive || spin_msg_error
  printf "$ZENPAKT_PROG_NAME: error : %s\n" "$@"
} >&2

error_print_v() {
  spin_inactive || spin_msg_error
  printf_v "$ZENPAKT_PROG_NAME: error : %s\n" "$@"
} >&2

#arg1 : continue arg2:the commands to be evaled
#store the callback or update it to new value
error_callback() {
  [[ $1 = continue ]] || return
  if [[ $2 ]] ; then
    _error_continue_callback=$2
  else
    _error_continue_callback=
  fi
}

#Check if either the identifier or the errno is already in use
#Usually used for debugging in case you override an existing value
#we should/can use readonly though that doesnot give the full picture.
#$1 $ERRNO $2 ERRIDENTIFIER
error_errno_isempty() {
  if [[ ${error[$1]} ]] ; then
    debug_msg "error[$1] already set"
    debug_msg "error[$1]=\"${error[$1]}\""
    return 1
  fi
  if [[ ${!2} ]] ; then
    debug_msg "$2 already set"
    debug_msg "$2=\"${!2}\""
    return 1
  fi
}
