#       inarray.bash
#       This file is part of zenpakt

#       Copyright 2011 Anshul Narula aka sunny <sunny_delhi18@yahoo.com>
#
#       Zenpakt is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation; either version 2 of the License, or
#       (at your option) any later version.
#
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#       MA 02110-1301, USA.

$DOC
=========
 inarray
=========

SYNOPSIS
--------

inarray [-PCE] [-m dimension] [-c col] [-I var] string list

DESCRIPTION
-----------

inarray looks through the arguments given as LIST and if an argument matches
STRING, it returns 0 or else false.

OPTIONS
-------
-P  treat STRING as a pattern
-C  do a case insensitive match, only if pattern matching is enabled
-E  use extended matching
-m Dimension  treat the list as false multi dimensional array of dimension DIMENSION
-c column  look only in column COLUMN of the array of dimension DIMENSION.The
          column and dimensions start from 1
-i var  Store the index of first match found in VAR

$ENDDOC

#This is the earlier implementation of inarray
#inarray is now a builtin and enabled after defining all the functions
#by _footer_.bash
inarray()
{
################
#LOCAL VARIABLES
################
local OPTIND=1 OPTARG opt || return 1
declare -A I || return 1
I=(
  [ind]=0
  [variable]=
  [tofind]=
  [dimension]=1
  [col]=1
  [i]=0
 )

###########
#OPTIONS
###########
while getopts :Ci:m: opt ; do
  case $opt in
    C)
      trap 'shopt -u nocasematch' RETURN
      shopt -s nocasematch
      ;;
    i)
      I[variable]=$OPTARG
      ;;
    m)
      if [[ $OPTARG = +([0-9]):+([0-9]) ]] ; then
        IFS=: read I[dimension] I[col] <<<"$OPTARG"
        ((${I[col]} <= 0 || ${I[col]} > ${I[dimension]})) && return 1
      else
        return 1
      fi
      ;;
    \?)
      #print -d "$FUNCNAME: unknown option -$OPTARG"
      return 1
      ;;
    :)
      #print -d "$FUNCNAME: -$OPTARG requires an argument"
      return 1
      ;;
  esac
done

#if we at least have VALUE to look for, even if list is an empty list
#we continue by shifting the parameters or else we give shift a value that
#will make it error
if ! shift $(($# > OPTIND ? OPTIND-1 : -1)) 2>/dev/null ; then
  #print -d "$FUNCNAME: no value provided to look for in the list"
  return 1
fi

I[tofind]=$1
shift

###################
#FIND THE ELEMENT
###################

#look for each element according to the dimension
#the the element index for c'th column of r'th row of an n dimensional array is
#   n*r + c
for (( I[ind]=I[i]*I[dimension]+I[col] ; I[ind] <= $# ; I[ind]=(++I[i])*I[dimension]+I[col] )) ; do
  [[ ${@:I[ind]:1} = "${I[tofind]}" ]] && I[i]=-1 && break
done

((I[i] < 0)) || return 1
[[ ${I[variable]} ]] || return 0
printf -v "${I[variable]}" '%s' "$((I[ind]-1))"
}

$$

