#       meta_getinstalledpkgname.bash
#       This file is part of Zenpakt

#       Copyright 2011 Anshul Narula aka sunny <sunny_delhi18@yahoo.com>
#
#       Zenpakt is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation; either version 2 of the License, or
#       (at your option) any later version.
#
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#       MA 02110-1301, USA.

$DOC

===============
 meta_find_pkg
===============

Synopsis
--------

**meta_find_pkg** *pattern* *var*

**meta_find_pkg_first** *pattern* *var*

**meta_find_pkg_unique** *pattern* *var*

**meta_find_pkg_glob** *pattern* *var*

**meta_find_pkg_glob_first** *pattern* *var*

**meta_find_pkg_glob_unique** *pattern* *var*

**meta_find_pkg_noglob** *pattern* *var*

**meta_find_pkg_noglob_first** *pattern* *var*

**meta_find_pkg_noglob_unique** *pattern* *var*

Description
-----------

**meta_find_pkg** searches for installed packages in database $ADM_DIR
with the given pattern. If pattern starts with a /, it is currently considered
as a pattern with a globbing * at the end(the behaviour will change in future)
and **meta_find_pkg_glob** is called.Otherwise, **meta_find_pkg_noglob** is
called.

**meta_find_pkg_first**  is like **meta_find_pkg** but it calls
**meta_find_pkg_glob_first** and **meta_find_pkg_noglob_first** instead with the
aim to store the first among many matches.

**meta_find_pkg_unique**  is like **meta_find_pkg** but it calls
**meta_find_pkg_glob_unique** and **meta_find_pkg_noglob_unique** instead with
the aim to store a match only if a unique/single match is found.

**meta_find_pkg_glob** searches for packages matching pattern with a globbing *
at the end and stores all matches in given array *var*.

**meta_find_pkg_glob_first** is like **meta_find_pkg_glob** but it stores the
first among various matches in the given *var*.

**meta_find_pkg_glob_unique** is like **meta_find_pkg_glob** but it stores the
a matched package name only if it was the only match found.

**meta_find_pkg_noglob** searches the database for a package matching the
pattern completely.It should match here for a package with entire basename
specified or for old style package name.If found, the var is set to the name of
the package.If not found, the pattern is considered as a package name with the
version,architecture and build number removed.The pattern is then matched for
installed package names which may match more than one package name in case more
than one version for the same package was installed.

**meta_find_pkg_noglob_first** is like **meta_find_pkg_noglob** but it stores
only first among all matches in case more than one match is found.

**meta_find_pkg_noglob_unique** is like **meta_find_pkg_noglob**  but it stores the
a matched package name only if it was the only match found.

Return Status
-------------

* 0  success
* 1  no match found
* 2  In case of meta_find_pkg_unique and functions called by it, if more than
     one match is found

$ENDDOC

$$

meta_find_pkg() {
  if [[ $1 = /* ]] ; then
    meta_find_pkg_glob "${1#/}" "$2" || return $?
  else
    meta_find_pkg_noglob "$@" || return $?
  fi
}

meta_find_pkg_first() {
  if [[ $1 = /* ]] ; then
    meta_find_pkg_glob_first "${1#/}" "$2" || return $?
  else
    meta_find_pkg_noglob_first "$@" || return $?
  fi
}

meta_find_pkg_unique() {
  if [[ $1 = /* ]] ; then
    meta_find_pkg_glob_unique "${1#/}" "$2" || return $?
  else
    meta_find_pkg_noglob_unique "$@" || return $?
  fi
}

meta_find_pkg_glob() {
    local _meta_find_pkg_glob

    shopt -s nullglob
    _meta_find_pkg_glob=( "$ADM_DIR/packages/$1"* )
    shopt -u nullglob

    if ((${#_meta_find_pkg_glob[@]})) ; then
      IFS=' ' read -a "$2" < <(
        printf '%q ' "${_meta_find_pkg_glob[@]##*/}"
	printf \\n
      )
    else
      return 1
    fi
}

meta_find_pkg_glob_first() {
    local _meta_find_pkg_glob_first

    shopt -s nullglob
    for _meta_find_pkg_glob_first in "$ADM_DIR/packages/$1"* ; do
      printf -v "$2" '%s' "${_meta_find_pkg_glob_first##*/}"
      shopt -u nullglob
      return 0
    done

    shopt -u nullglob
    return 1
}

meta_find_pkg_glob_unique() {
    local _meta_find_pkg_glob_unique

    shopt -s nullglob
    _meta_find_pkg_glob_unique=( "$ADM_DIR/packages/$1"* )
    shopt -u nullglob

    ((${#_meta_find_pkg_glob_unique[@]})) || return 1
    ((${#_meta_find_pkg_glob_unique[@]} != 1)) && return 2
    printf -v "$2" '%s' "${_meta_find_pkg_glob_unique##*/}"
}

meta_find_pkg_noglob() {
  #check for a perfect match first
  if [[ -f $ADM_DIR/packages/$1 ]];then
    unset "$2"
    printf -v "$2" '%s' "$1"
  else
    #Assume the package is given as basename
    local _meta_find_pkg_noglob

    shopt -s nullglob
    _meta_find_pkg_noglob=( "$ADM_DIR/packages/$1"@(-+([^-])-+([^-])-+([^-])) )
    shopt -u nullglob

     # no package installed so return 1
    ((${#_meta_find_pkg_noglob[@]})) || return 1

    IFS=' ' read -a "$2" < <(
      printf '%q ' "${_meta_find_pkg_noglob[@]##*/}"
      printf \\n
    )
  fi
}

meta_find_pkg_noglob_first() {
  #check for a perfect match first
  if [[ -f $ADM_DIR/packages/$1 ]];then
    unset "$2"
    printf -v "$2" '%s' "$1"
  else
    #Assume the package is given as basename
    local _meta_find_pkg_noglob_first

    shopt -s nullglob
    for _meta_find_pkg_noglob_first in "$ADM_DIR/packages/$1"@(-+([^-])-+([^-])-+([^-]))
    do
      shopt -u nullglob
      printf -v "$2" '%s' "${_meta_find_pkg_noglob_first##*/}"
      return 0
    done

    shopt -u nullglob

    return 1
  fi
}

meta_find_pkg_noglob_unique() {
  #check for a perfect match first
  if [[ -f $ADM_DIR/packages/$1 ]];then
    unset "$2"
    printf -v "$2" '%s' "$1"
  else
    #Assume the package is given as basename
    local _meta_find_pkg_noglob_unique

    shopt -s nullglob
    _meta_find_pkg_noglob_unique=( "$ADM_DIR/packages/$1"@(-+([^-])-+([^-])-+([^-])) )
    shopt -u nullglob

     # no package installed so return 1
    ((${#_meta_find_pkg_noglob_unique[@]})) || return 1
    ((${#_meta_find_pkg_noglob_unique[@]} != 1)) && return 2

    printf -v "$2" '%s' "${_meta_find_pkg_noglob_unique##*/}"
  fi
}
