#       meta_init_db.bash
#       This file is part of zenpakt
#
#       Copyright 2010 Anshul Narula aka sunny <sunny_delhi18@yahoo.com>
#
#       Zenpakt is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation; either version 2 of the License, or
#       (at your option) any later version.
#
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#       MA 02110-1301, USA.


$DOC
==============
 meta_init_db
==============

--------------------------
 initialize meta database
--------------------------

SYNOPSIS
--------

**meta_init_db**
**meta_init_tmpdir**
**meta_rm_tmpdir**

DESCRIPTION
-----------

It initializes directories required for package management.It requires ROOT to
be set.Following Global variables are set:

ADM_DIR         the admininstration dir (ROOT/var/log)
ZENPAKT_TMPDIR  the temporary directory (ADM_DIR/setup/tmp/random-numbers)

Following Directories are created under ADM_DIR:

 *  packages/
 *  scripts/
 *  removed_packages/
 *  removed_scripts/
 *  setup/tmp
 *  partial_packages/
 *  ZENPAKT_TMPDIR

The partial_packages directories should hold the package index for packages
whose install/upgrade was interrupted unexpectedly.You should reinstall or
remove those packages.The information for these shall be obtainable via via pkgdb
--partial partial|-p partial in future

RETURN STATUS
-------------

 *  0   success
 *  >0  error

$ENDDOC

$$

ADM_DIR=

meta_init_db() {
  #Create ADM_DIR
  ADM_DIR=$ROOT/var/log
  mkdir -p "$ADM_DIR"
  [[ -d $ADM_DIR ]] || {
    printf 'failed to initialize package database: %s does not exist' "$ADM_DIR"
    return 1
  }

  local dir

  #Initialize subdirectories required under ADM_DIR
  for dir in packages removed_packages partial_packages removed_scripts \
    partial_scripts scripts setup ; do

    if [[ ! -d $ADM_DIR/$dir ]];then
      rm -rf -- "$ADM_DIR/$dir"
      if ! mkdir -m 755  "$ADM_DIR/$dir" 2>/dev/null ; then

	printf 'failed to initialize package database: %s couldnot be created' \
	  "$dir" >&2
	return 1
      fi

    fi
  done

  #create temporary directory
  if [[ ! -d $ADM_DIR/setup/tmp ]] ; then
    rm -rf -- "$ADM_DIR/setup/tmp"

    if ! mkdir -m 700 "$ADM_DIR/setup/tmp" ; then
      printf 'failed to create directory %s\n' "$ADM_DIR/setup/tmp" >&2
      return 1
    fi

  else #cleanup coredump
    shopt -s nullglob dotglob
    rm -rf "$ADM_DIR/setup/tmp/"*
    shopt -u nullglob dotglob
  fi

  readonly ADM_DIR
}

meta_init_tmpdir() {
  mktemp_init_tmpdir "$ADM_DIR/setup/tmp" || {
    printf 'failed to create temporary directory under %s\n' "$ADM_DIR/setup/tmp" >&2
    return 1
  }
}

meta_rm_tmpdir() {
  mktemp_rm_tmpdir
}
