#       pkg_desc.bash
#       This file is part of zenpakt

#       Copyright 2010 Anshul Narula aka sunny <sunny_delhi18@yahoo.com>
#
#       Zenpakt is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation; either version 2 of the License, or
#       (at your option) any later version.
#
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#       MA 02110-1301, USA.

$DOC
==========
 pkg_desc
==========

Synopsis
--------

pkg_desc package compression basename name csize usize full_desc desc [ md5sum ]

Description
-----------

get description, compressed and uncompressed size and md5sum (if $MD5SUM is non-
zero) for the package which should be uncompressed using compression utility.The
basename and name of package are used for grepping description.

The variables specified as arguments store resulting values:

csize  compressed size
usize  uncompressed size
full_desc  desc along with basename or name
desc  desc without basename/name
md5sum  the md5sum

$ENDDOC

$$

pkg_desc() {
  local _pkg_desc_size _pkg_desc_usize _pkg_desc_ _pkg_desc_full

  #compressed size of the package
  _pkg_desc_csize=$(wc -c < "$1" )
  if ((_pkg_desc_csize >  1024 )) ; then
    _pkg_desc_csize=$((_pkg_desc_csize/1024))K
  else
    _pkg_desc_csize+=B
  fi

  printf -v "$5" '%s' "$_pkg_desc_csize"

  if  _pkg_desc_usize=$( "$2" -dc "$1" | wc -c ) 2>/dev/null ; then
    if ((_pkg_desc_usize > 1024)) ; then
      _pkg_desc_usize=$((_pkg_desc_usize/1024))K
    else
      _pkg_desc_usize+=B
    fi
  fi

  printf -v "$6" '%s' "$_pkg_desc_usize"

  #First check for .txt file next to the package:
  if [[ -r ${1%.*}.txt ]] && {
    grep -q "^$3:" "${1%.*}.txt" ||
    grep -q "^$4:" "${1%.*}.txt"
  };then
    _pkg_desc_=$(< "${1%.*}.txt")

  elif ! _pkg_desc_=$("$2" -dc "$1" | tar -xOf - install/slack-desc 2>/dev/null);then
    printf -v "$7" '%s'
    printf -v "$8" '%s\n' "<no description found>"
    return 0
  fi

  _pkg_desc_full=$( echo "$_pkg_desc_" | grep "^$3:")
  _pkg_desc_full+=$( echo "$_pkg_desc_" | grep "^$4:")
  printf -v "$7" '%s' "$_pkg_desc_full"

  _pkg_desc_full=${_pkg_desc_full#$3:}
  _pkg_desc_full=${_pkg_desc_full#$4:}
  _pkg_desc_full=${_pkg_desc_full//$'\n'$3:/$'\n'}
  _pkg_desc_full=${_pkg_desc_full//$'\n'$4:/$'\n'}

  printf -v "$8" '%s' "$_pkg_desc_full"
  (( MD5SUM )) &&  read "$9" _ <<< "$(md5sum "$1" )"
  return 0
}
