#       pkg_file_list.bash
#       This file is part of zenpakt
#
#       Copyright 2010 Anshul Narula aka sunny <sunny_delhi18@yahoo.com>
#
#       Zenpakt is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation; either version 2 of the License, or
#       (at your option) any later version.
#
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#       MA 02110-1301, USA.

$DOC
===============
 pkg_file_list
===============

Synopsis
--------

pkg_file_list D|I|DI|A package compression

Description
-----------

print a list of newline separated files as produced by tar -tf for the given
package.If you need to store it in a file just redirect the output.In case,
you want to store it in an array, use mapfile or a while read loop.
The first argument serves as a filter for list generated

*  D  do not list directories(ending with trailing /)
*  I  do not list contents of install/
*  DI  a combination of D and I
*  A  list all

$ENDDOC

$$

pkg_file_list() {
  case $1 in
    D)
      "$3" -dc "$2" | tar -tf - | grep -v '/$'
      ;;
    I)
      "$3" -dc "$2" | tar -tf - | grep -v '^install/'
      ;;
    DI)
      "$3" -dc "$2" | tar -tf - | grep -v '/$' | grep -v '^install/'
      ;;
    A)
      "$3" -dc "$2" | tar -tf -
      ;;
  esac
} 2>/dev/null
