#       Copyright 2010 Anshul Narula aka sunny <sunny_delhi18@yahoo.com>
#
#       This file is part of zenpakt

#       Zenpakt is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation; either version 2 of the License, or
#       (at your option) any later version.
#
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#       MA 02110-1301, USA.

$DOC

======
 spin
======

Synopsis
--------

**spin_msg** *msg* [ *style* ]
**spin_msg_error** [ *msg* ]
**spin_msg_success** [ *msg* ]
**spin_msg_end**
**spin_inactive**
**spin** *msg* [ *style* ]

Description
-----------

**spin_msg** prints the given message depending on the value of *RAW*. If *RAW*
is non-zero, it simply prints the message without a terminating newline.If *RAW*
is zero, it calls spin with its arguments in the background and stores its pid.
In both cases the last message with which it was called is stored and the state
of spin is made active.

**spin_msg_error** and **spin_msg_success** can be called subsequent to a call to
**spin_msg** to notify error or success.**spin_msg_error** prints failed or *msg*
specified.**spin_msg_success** prints done or *msg*.
Both call **spin_msg_end**

A call to **spin_msg_end** is necessary after a call to **spin_msg** either directly
or indirectly via **spin_msg_error** or **spin_msg_success**.It kills the background
spin call , resets the pid variable and state variable.

**spin_inactive** returns nonzero if **spin_msg** was called but a subsequent call
to **spin_msg_end** was still pending.

**spin** starts an infinite loop printing the given message with a spinner.The
*style* argument tells it which style of spinner to use:
  1  bars
  2  bubble
  3  cross
  4  arrows

The default spinner is ZP_SPIN_STYLE if set or 1.

All spin functions respect value of **VERBOSE**

$ENDDOC

$$

_spin_last_msg=
_spin_pid=
_spin_state=0

#spin_msg msg [style]
spin_msg () {
  # if raw is nonzero spin is not needed
  if ((RAW)) ; then
    echon_v "$1"
  else
    spin "$@" &
    _spin_pid=$!
  fi
  _spin_last_msg=$1
  _spin_state=1
} 2>/dev/null

#Using SIGTERM with kill leads to race condition when trap is set on TERM by
#calling scripts( see trap.c in bash source code ).
spin_msg_end () {
  _spin_state=0
  ((RAW)) && return
  disown "$_spin_pid"
  kill -HUP "$_spin_pid"
  _spin_pid=
}

spin_inactive () { return "$_spin_state" ;}

spin_msg_success () {
  spin_msg_end
  printf_v '\r * %s  %s\n' "${_spin_last_msg}" \
    "${TXT_bold}${COL_green}${1:-done}${TXT_reset}"
}

spin_msg_error () {
  spin_msg_end
  printf_v "\r * %s  %s\n" "${_spin_last_msg}" \
    "${TXT_bold}${COL_red}${1:-failed}${TXT_reset}"

}

#spin msg [style]
spin () {
  local i=0 spin bars bubble cross arrow sleep=0.5

  #the various spin styles
  #the default spin style is dependant on env var ZP_SPIN_STYLE if set or else 1
  spin=1
  bars=('-' \\  '|' '/')
  bubble=('.' 'o' 'O' 'o')
  cross=('+' 'x' '+' 'x')
  arrow=('> >' '>> ' '>>>' ' >>')

  if [[ $2 = [1234] ]] ; then
    spin=$2
  elif [[ $ZP_SPIN_STYLE = [1234] ]] ; then
    spin=$ZP_SPIN_STYLE
  fi

  case $spin in
    1)
      spin=("${bars[@]}")
      ;;
    2)
      spin=("${bubble[@]}")
      ;;
    3)
      spin=("${cross[@]}")
      ;;
    4)
      spin=("${arrow[@]}")
      ;;
  esac

  while true ; do
    printf_v "\r%s%s%s %s" "${TXT_bold}${COL_cyan}" \
      "${spin[i++%${#spin[@]}]}" "${TXT_reset}" "$1"
    sleep "$sleep"
  done
}
