#       Copyright 2010 Anshul Narula aka sunny <sunny_delhi18@yahoo.com>
#
#       This file is part of zenpakt

#       Zenpakt is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation; either version 2 of the License, or
#       (at your option) any later version.
#
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#       MA 02110-1301, USA.

$DOC
====================
 installpkg_cleanup
====================

-------------------------------
 clean up the mess before exit
-------------------------------

Description
-----------

This should be called via a trap set on EXIT.
First it removes $ROOT/install if it exists.Then if backup tar exists and
install was not done properly, removes partially installed package and then
restores from backup.If install was done it just removes the backup.Then it
removes the temporary directory.If exit status is other than $ERR_LOCKED it
calls lock -r to remove the lock.

Since behaviour of cleanup is based on $ROOT, trap should be set after ROOT is
determined from environment and argument parsing, ie after a call to
installpkg_parseargs.Also, since it tends to remove TMPDIR, it should be set
after a call to meta_init_db.Also, as it tends to remove the lock and since lock
maynot be the first thing checked it trap should be set immediately before lock
is called.

Since meta_init_db is called after lock is set (we mess with file system after
creating a lock).Above conditions cannot be met.Ensuring that TMPDIR has an
empty value before defining meta_init_db allows and not setting TMPDIR anywhere
until meta_init_db is called allows setting the trap right before calling lock
and allows meta_init_db to be called after lock by ensuring that we do not
remove anything else stored in environmental TMPDIR in case trap is triggered
before a call to meta_init_db.

$ENDDOC

$$
installpkg_cleanup() {
  #remove install/ if it exists
  rm -rf "$ROOT"/install

  #if paranoid is 1
  if ((PARANOID)) && [[ $MODE != warn ]] ; then

    #if the state is 0 we were interrupted while installation of a package was
    #proceeding which means we may need to cleanup.
    #If the backup exists and we are exiting, installation was not done
    #Which may mean we were interrupted while installation was still in progress
    #Because immediately after successfull/unsuccessful installation and
    #configuration we restore from the backup and/or just remove it.
    #If backup doesnot exist, we already finished the installation.There is
    #a race between the removal of backup and switching of the state back to 1.
    #Since backup is a temporary file, its existence is used instead of
    #checking wether the doinst script was moved to its location which would
    #be the actual last step of installation
    #
    #State is set to 0 after backup is done but before installation proceeds
    #Combining the two gives us a less racy way to determine wether installation
    #was interrupted or not.Race condition still exists though.
    if installpkg_state && installpkg_bkup_exists; then

      local basename
      installpkg_state_basename basename


      #if database was written but installation was not done:
      #We move the database to the partial_packages directory
      if [[ -e $ADM_DIR/packages/$basename ]] ; then
        mv "$ADM_DIR/"{,partial_}packages/"$basename"

      #If database was not written but temporary file was created, move it
      #to partial database, if that fails just create an empty database file
      #with no content only if a file doesnot already exist in the partial
      #database, there is a chance that we actually called installpkg_install
      #which failed and we called installpkg_move_db_partial which succeeded
      #but didnot yet restore from the backup while we were interrupted.
      elif ! installpkg_move_db_partial "$basename" ; then
        [[ -e "$ADM_DIR/partial_packages/$basename" ]] ||
        > "$ADM_DIR/partial_packages/$basename"
      fi

      #remove the doinst script if its there
      rm -f "$ADM_DIR/scripts/$basename"

      #restore from the backup
      installpkg_bkup_restore
    fi
  fi

  #the temporary directory can now be removed
  meta_rm_tmpdir

  #if last set exit status is lock failure, we do not remove lock
  local exitstatus
  error_exit_status exitstatus
  ((exitstatus == ERR_LOCKED)) || lock -r

}
