#       Copyright 2010 Anshul Narula aka sunny <sunny_delhi18@yahoo.com>
#
#       This file is part of zenpakt

#       Zenpakt is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation; either version 2 of the License, or
#       (at your option) any later version.
#
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#       MA 02110-1301, USA.

#HELP
#installpkg_install package [bkupdir]
#if backupdir is given it is used for backups and is not cleared after installation
#else it makes its own bkup dir $installpkg_bkup_dir, it is not removed on successful installation(so that installpkg_configure can use it) but is cleared
#if 3rd argument is given and is 0 then configuration of package is not done
#return codes:
# 2: error writing database
# 1: error getting filelist
# 3: overwrite not allowed
# 4: error removing symlink
# 5: error exploding package
#requires pkg_getpkgdescription to be called first
#ENDHELP
#the metadatabase is written in two steps....race condition needs to be fixed

#Information needed
#package
#compression
#basename
#csize
#usize
#full desc-->name of the variable
#paranoid level

$DOC

====================
 installpkg_install
====================

SYNOPSIS
--------

installpkg_install package compression basename csize usize fulldesc [md5sum]

installpkg_install_state

installpkg_move_db_partial

DESCRIPTION
-----------

installpkg_install
~~~~~~~~~~~~~~~~~~

Install the given package.The various arguments are:

Package  the fullpath to package to be installed
compression  The compression utility to use
basename  the basename of the package
csize  the compressed size
usize  the uncompressed size
fulldesc  the variable name holding the full desc which should be used via indirection
md5sum  the md5sum if to be recorded or an empty argument

installpkg_install_state
~~~~~~~~~~~~~~~~~~~~~~~~

The global state variable for a call to installpkg_install is set to zero
initially.It is set to 1 when installpkg_install is called and back to zero when
it returns wether successfully or unsuccessfully.If the calling script is
interrupted while installpkg_install was called but did not yet return, the value
of this variable remains 1.This can be checked by a call to
installpkg_install_state which returns the value of this variable.If the returned
status is 1, a call to installpkg_move_db_partial should be done and also bkup
should be restored.

installpkg_move_db_partial
~~~~~~~~~~~~~~~~~~~~~~~~~~

If an error occurs while moving the temporary file to the actual database
directory, the database file should be moved to the partial database directory.
This is done by installpkg_move_db_partial.

$ENDDOC

$$

#_installpkg_install_writedb dbfile package compression basename csize usize
#                            fulldesc [md5sum]
#called by installpkg_install
#All arguments to installpkg_install are passed follwing dbfile so that
#arguments are increased by one position
#$1  dbfile
#$2  package
#$3  compression (not used here)
#$4  basename
#$5  csize
#$6  usize
#$7  fulldesc
#$8  md5sum
_installpkg_install_writedb() {
   printf 'PACKAGE NAME:     %s\n' "$4"
   printf 'COMPRESSED PACKAGE SIZE:     %s\n' "$5"
   printf 'UNCOMPRESSED PACKAGE SIZE:     %s\n' "$6"
   printf 'PACKAGE LOCATION: %s\n'  "$2"
   [[ $8 ]] && printf 'PACKAGE MD5SUM: %s\n' "$8"
   printf 'PACKAGE DESCRIPTION:\n%s\nFILE LIST:\n' "${!7}"
   #make writing meta database a one shot operation so that we can call
   #compression -dc | tar | writemetadb
   #all together
   cat
} > "$1"

#zero means installpkg_install wasn't called or call to it is over
#1 means installpkg_install is under call
_installpkg_install_state=0

#$1  package
#$2  compression
#$3  basename
#$4  csize
#$5  usize
#$6  fulldesc
#$7  md5sum
installpkg_install() {

  local _installpkg_install_dbfile

  #set trap to remove the extracted install/ because
  #we donot need it after install and doisnt/slack-desc
  #for next package may be confused for current package's
  #in case they dont exist for later, if install/ is not removed
  #Also remove the backed up tar file and the temporary dbfile
  trap 'rm -rf "$ROOT/install" _installpkg_install_state=0; trap RETURN' RETURN

  _installpkg_install_state=1

  #create a temporary database file
  mktemp _installpkg_install_dbfile || return "ERR_CREAT_TMP"

  #if we remove -U option we can safely use newer versions of tar without the bug of
  #removing symlinked directories
  if ( set -o pipefail
    cd "$ROOT" &&
    "$2" -dc | tar -xlUpvf - |
    _installpkg_install_writedb "$_installpkg_install_dbfile" "$@" ) < "$1" #2>/dev/null ;
  then
    mv "$_installpkg_install_dbfile" "$ADM_DIR/packages/$3" || return "$ERR_WRITE_DB"

  else
    return "$ERR_INST_EXTRACT"
  fi
  _installpkg_install_dbfile=
}

installpkg_install_state() {
  return "$_installpkg_install_state"
}

installpkg_move_db_partial() {
  [[ -f $_installpkg_install_dbfile ]] || {
    _installpkg_install_dbfile=
    return 1
  }
  mv "$_installpkg_install_dbfile" "$ADM_DIR/partial_packages/$1" || return 1
  _installpkg_install_dbfile=
}
