#       Copyright 2010 Anshul Narula aka sunny <sunny_delhi18@yahoo.com>
#
#       This file is part of zenpakt

#       Zenpakt is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation; either version 2 of the License, or
#       (at your option) any later version.
#
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#       MA 02110-1301, USA.

$DOC
======================
 installpkg_parseargs
======================

Synopsis
--------

**installpkg_parseargs**

Description
-----------

It defines a local getargs array containing the various arguments accepted by
installpkg and a helping function _installpkg_parseargs_set_mode_ to set and
ensure the exclusivity of variable MODE.Then it redefines itself ,i.e. defines
another function named installpkg_parseargs which is serves as callback function
to getargs setting various variables.

If getargs returns unsuccessfully due to general option parsing errors or due to
unsuccessful call to callback function or GETARGS_IND is more than $# ( ie no
mass arguments were specified), the variables VERBOSE, RAW and ASK are redefined
as they may affect the output and input behaviour while exiting.After which,
installpkg_usage is called and functions returns non zero.

If call to getargs was successful and mass arguments are present, the variables
ASK RAW MD5SUM ROOT overwrite MODE PARANOID VERBOSE are made readonly.MODE may
already be readonly in case an argument setting a mode such as --warn was
specified via a call to _installpkg_parseargs_set_mode_ through callback
function of getargs.However, if no mode option was given, MODE with its default
value is made readonly for rest of execution of the script.

On return, wether successfully or unsuccessfully, redefined installpkg_parseargs
and _installpkg_parseargs_set_mode_ are unset, so that no function named
installpkg_parseargs and _installpkg_parseargs_set_mode_ exists after a single
call to installpkg_parseargs.This function, thus can be used only once.

For further details, refer doc for getargs.

$ENDDOC

$$

installpkg_parseargs() {

  local getargs

  getargs=(
    w    warn         -  1
    R    raw          -  1
    a    ask          -  1
    V    no-verbose   -  1
    m    md5sum       -  1
    r    root         :  1
    O    no-overwrite -  1
    f    fast         -  1
  )

  _installpkg_parseargs_set_mode_() {
    if ! readonly MODE=$1 2>/dev/null ; then
      error_print "mutually exclusive modes: $MODE and $1"
      return 1
    fi
  }

  #Redefine installpkg_parseargs to a function which can be used as a callback
  #function by getargs, we use the same function name because we need to parse
  #only once.
  installpkg_parseargs() {
    case "$2" in
      w|warn)
        _installpkg_parseargs_set_mode_ warn || return 1
        RAW=1
        ;;

      V|no-verbose)
        _installpkg_parseargs_set_mode_ quiet || return 1
        VERBOSE=0 RAW=1
        ;;

      ask|a)
        _installpkg_parseargs_set_mode_ ask || return 1
        ASK=1
        ;;

      raw|R)
        _installpkg_parseargs_set_mode_ raw || return 1
        RAW=1
        ;;

      md5sum|m)
        MD5SUM=1
        ;;

      fast|f)
        PARANOID=0
        ;;

      root|r)
        if [[ ! -d $3 ]] ; then
          printf '%s\n' "root directory $3 does not exist" >&2
          return 1
        fi
        fullpath "$3" ROOT
        ;;

      no-overwrite|O)
        INSTALLPKG_OVERWRITE=0
        ;;

      version)
        version 'GPL2+'
        exit 0
        ;;

      help)
        usage 0 installpkg
        exit 0
        ;;
    esac
  }

  trap 'unset -f _installpkg_parseargs_set_mode_ installpkg_parseargs' RETURN

  #In case of an error the displaying of specific error messages are
  #handled inside the callback function.However certain errors like repeating
  #an option more than allowed or using an unknown option makes getargs return
  #non-zero without a call to the callback function.Therefore the final resetting
  #of certain variables must be done here and not inside the callback function.
  if ! getargs installpkg_parseargs "$@" || (($GETARGS_IND > $#)) ; then
    VERBOSE=1 ASK=0 RAW=1
    usage 1
    return 1
  fi

  readonly ASK RAW MD5SUM ROOT INSTALLPKG_OVERWRITE MODE PARANOID VERBOSE ||
  return 0
}
