#       Copyright 2010 Anshul Narula aka sunny <sunny_delhi18@yahoo.com>
#
#       This file is part of zenpakt

#       Zenpakt is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation; either version 2 of the License, or
#       (at your option) any later version.
#
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#       MA 02110-1301, USA.

$DOC

========================
 removepkg_move_dbentry
========================

SYNOPSIS
--------

**removepkg_move_dbentry** package...

**removepkg_move_dbentry_status**

**removepkg_unmove_dbentry**


DESCRIPTION
-----------

removepkg_move_dbentry
~~~~~~~~~~~~~~~~~~~~~~

Moves database entry and doisnt.sh if one exists of given package names
from $ADM_DIR/packages and $ADM_DIR/scripts to $ADM_DIR/removed_packages and
$ADM_DIR/removed_scripts respectively.

In case PARANOID is nonzero, the status is set to 0(initially the status is 1)
after successful creation of a temporary file.If temporary file creation fails
it returns ERR_CREAT_TMP.Files successfuly moved are stored in the temporary
file list.In case an error occurs while moving, it immediately returns
ERR_MOVE_DB. If the moving of all database entry succeeds, status is reset to 1.

In case PARANOID is 0, no temporary file is created and no list is stored.The
status remains 1 and is not changed.In case error occurs while moving it returns
ERR_MOVE_DB.

removepkg_move_dbentry_status
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

Returns the status of a call to removepkg_move_dbentry.If the status is 0,
removepkg_move_dbentry was called and PARANOID was non zero and there was an
error while moving files or there was an interruption via a signal.As such, if
status is 0, we should call removepkg_unmove_dbentry and removepkg_bkup_restore
from cleanup function.

removepkg_unmove_dbentry
~~~~~~~~~~~~~~~~~~~~~~~~

Moves back all the moved database entry and doinst.sh by removepkg_move_dbentry
back to their place.

$ENDDOC

$$

#the status for call to removepkg_move_dbentry
_removepkg_move_dbentry_status=1

#the list of files moved
_removepkg_move_dbentry_file=

removepkg_move_dbentry() {

  local i

  if ((PARANOID)) ; then

    mktemp _removepkg_move_dbentry_file || return "$ERR_CREAT_TMP"
    _removepkg_move_dbentry_status=0

    for i do

      mv "$ADM_DIR/packages/$i" "$ADM_DIR/removed_packages/" ||
      return "$ERR_MOVE_DB"

      printf '%s\0' "$ADM_DIR/removed_packages/$i" \
        >> "$_removepkg_move_dbentry_file"

      if [[ -f $ADM_DIR/scripts/$i ]] ; then

	mv "$ADM_DIR/scripts/$i" "$ADM_DIR/removed_scripts/" ||
	return "$ERR_MOVE_DB"

	printf '%s\0' "$ADM_DIR/removed_scripts/$i" \
	  >> "$_removepkg_move_dbentry_file"
      fi

    done

    _removepkg_move_dbentry_status=1

  else

    for i do

      mv "$ADM_DIR/packages/$i" "$ADM_DIR/removed_packages/" ||
      return "$ERR_MOVE_DB"

      [[ $ADM_DIR/scripts/$i ]] && {
        mv "$ADM_DIR/scripts/$i" "$ADM_DIR/removed_scripts/" ||
	return "$ERR_MOVE_DB"
      }

    done

  fi

}

removepkg_move_dbentry_status() {
  return "$_removepkg_move_dbentry_status"
}

removepkg_unmove_dbentry() {
  [[ -f $_removepkg_move_dbentry_file ]] || return
  while read -rd '' ; do
    mv "$REPLY" "$ADM_DIR/${REPLY#$ADM_DIR/removed_}"
  done < "$_removepkg_move_dbentry_file"
  rm "$_removepkg_move_dbentry_file"
} 2>/dev/null

