#       Copyright 2010 Anshul Narula aka sunny <sunny_delhi18@yahoo.com>
#
#       This file is part of zenpakt

#       Zenpakt is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation; either version 2 of the License, or
#       (at your option) any later version.
#
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#       MA 02110-1301, USA.

$$

removepkg_parseargs() {

  local getargs removeall

  #REMOVEALL has three values it can be only 0, ie in case of --warn
  removeall=(
    [0]="--warn|-w"
    [1]="--remove-all|-A"
    [2]="--unique|-u"
  )

  getargs=(
    w    warn        -  1
    a    ask         -  1
    R    raw         -  1
    V    no-verbose  -  1
    f    fast        -  1
    r    root        :  1
    m    max         :  1
    A    remove-all  -  1
    u    unique      -  1
  )

  _removepkg_parseargs_set_mode_() {
    if ! readonly MODE=$1 2>/dev/null; then
      error_print "mutually exclusive modes: $MODE and $1"
      return 1
    fi
  }

  _removepkg_parseargs_set_removeall_() {
    if ! readonly REMOVEALL=$1 2>/dev/null ; then
      error_print \
        "incompatible options: $2 and ${removeall[REMOVEALL]} cannot be used together"
      return 1
    fi
  }

  removepkg_parseargs() {
    case $2 in
      warn|w)
        _removepkg_parseargs_set_mode_ warn || return 1
        _removepkg_parseargs_set_removeall_ 0 "$1$2" 2>/dev/null || return 1
        RAW=1 VERBOSE=1
        ;;

      no-verbose|V)
        _removepkg_parseargs_set_mode_ quiet || return 1
        VERBOSE=0 RAW=1
        ;;

      ask|a)
        _removepkg_parseargs_set_mode_ ask || return 1
        ASK=1 RAW=0
        ;;

      raw|R)
        _removepkg_parseargs_set_mode_ raw || return 1
        RAW=1
        ;;

      remove-all|A)
        _removepkg_parseargs_set_removeall_ 1 "$1$2" || return 1
        ;;

      unique|u)
        _removepkg_parseargs_set_removeall_ 2 "$1$2" || return 1
        ;;

      max|m)
        if [[ $3 = *[^0-9]* ]] || (($3 < 0)) ; then
          printf '%s\n' "$3 is not a valid value for $1$2" >&2
          return 1
        fi
        REMOVEPKG_MAXRM=${ARG_OPTARG[count]}
        ;;

      fast|f)
        PARANOID=0
        ;;

      root|r)
        if [[ ! -d $3 ]] ; then
          printf '%s\n' "the specified root directory $ROOT does not exist" >&2
          return 1
        fi
        fullpath "$3" ROOT
        ;;

      version)
        version 'GPL2+'
        exit 0
        ;;

      help)
        usage 0 removepkg
        exit 0
        ;;
    esac
  }


  if ! getargs removepkg_parseargs "$@" || (($GETARGS_IND > $#)) ; then
    ASK=0 RAW=1 VERBOSE=1
    usage 1
    return 1
  fi

  readonly MODE RAW REMOVEPKG_MAXRM REMOVEPKG_MAXLIST \
    ASK ROOT REMOVEALL PARANOID VERBOSE || return 0
}
