#       Copyright 2010 Anshul Narula aka sunny <sunny_delhi18@yahoo.com>
#
#       This file is part of zenpakt

#       Zenpakt is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation; either version 2 of the License, or
#       (at your option) any later version.
#
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#       MA 02110-1301, USA.

$DOC

===================
 removepkg_remove
===================

SYNOPSIS
--------

**removepkg_remove_files**

**removepkg_bkup_and_remove_files**

**removepkg_remove_files_status**

**removepkg_bkup_and_remove_files_state**

**removepkg_remove_cats**

**removepg_remove_dirs**

**removepkg_remove_dirs_print_skipped**

**removepkg_move_dbentry**

**removepkg_move_dbentry_status**

**removepkg_unmove_dbentry**

**removepkg_restore**

DESCRIPTION
-----------

All the functions read from stdin and do not use any arguments.

removepkg_remove_files
~~~~~~~~~~~~~~~~~~~~~~

reads a newline delimited list of backslash quoted files, filters directories
and files under install/ and then converts them to non escaped null delimited
file list and removes them one by one and echoes them if VERBOSE is non-zero.If
PARANOID is non zero it just calls removepkg_bkup_and_remove_files.If a file
cannot be removed it returns ERR_REMOVE_FILES

removepkg_bkup_and_remove_files
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

It takes the same input and does the same as **removepkg_remove_files** but
instead of removing them one by one, it creates a backup tar using bkup_rm
while removing the files at the same time.If creation of temporary backup tar
fails it returns ERR_CREAT_TMP and if removing or backing up fails it returns
ERR_REMOVE_FILES.

removepkg_bkup_and_remove_files_state
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

returns the state set by removepkg_bkup_and_remove_files.The state is always 1.
If the removepkg_bkup_and_remove_files is called state is set to zero
if temporary ball creation succeeds .It is set back to 1 in the end only after
successful call to bkup_rm.In case of an interruption due to a signal or error
while backing up/removing files causing removepkg_bkup_and_remove_files to return
immediately, the state remains zero which can be used to determine if we need to
call removepkg_restore.

removepg_remove_dirs,removepkg_remove_dirs_print_skipped
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

filters directories from list read from stdin and tries to remove them if they
are empty.Directories which cannot be removed are stored in a temporary file.
The contents of this file are displayed in escaped version by removepkg_remove_
dirs_print_skipped which also removes the temporary file subsequently.

removepkg_remove_cats
~~~~~~~~~~~~~~~~~~~~~

read from stdin a newline delimited list like others but removes cat files
created by man(1) if found.

removepkg_restore
~~~~~~~~~~~~~~~~~

restores from the temporary tar created by removepkg_bkup_and_remove_files and
subsequently remove the backup tar file.

$ENDDOC

$$

removepkg_remove_files() {
  if ((PARANOID)) ; then
    removepkg_bkup_and_remove_files
  else
    local REPLY

    grep -v '/$' | grep -v '^install/' | convesc |
    while read -rd '' ; do
      print_v "$REPLY"
      rm -f -- "$ROOT/$REPLY" || return "$ERR_REMOVE_FILES"
    done
  fi
}

#the backup tar ball
_removepkg_bkup_and_remove_files_bkup=

#the state of removal; it is initialized to zero after successful creation of
#temporary tar file and set back to 1 after successful call to
#removepkg_bkup_and_remove_files is done;In case the function returns before
#it is set back to 1 due to failure of backup and removal or in case the script
#is interrupted during the process due to a signal the state remains 0 which
#can be used to determine if a restoration is needed or not in the cleanup
#function.If this function is not called either because the script exited
#before it could call it or because PARANOID was 0, the state remains 1.As such,
#checking the state via removepkg_bkup_and_remove_files_state can be used to
#determine if removepkg_restore should be called or not.
_removepkg_bkup_and_remove_files_state=1

#remove files (not directories)
removepkg_bkup_and_remove_files() {
  mktemp _removepkg_bkup_and_remove_files_bkup || return "$ERR_CREAT_TMP"

  _removepkg_bkup_and_remove_files_state=0

  grep -v '/$' | grep -v '^install/' | convesc |
  bkup_rm "$_removepkg_bkup_and_remove_files_bkup" "$ROOT" ||
  return "$ERR_REMOVE_FILES"

  _removepkg_bkup_and_remove_files_state=1
  return 0
}

removepkg_bkup_and_remove_files_state() {
  return "$_removepkg_bkup_and_remove_files_state"
}

#the file containing list of directories not removed because it was not empty
_removepkg_bkup_and_remove_dir_skipped=

removepkg_remove_dirs() {
  local REPLY
  mktemp _removepkg_bkup_and_remove_dir_list

  grep '/$' | grep -v '^install/' | grep -v '^\./$' |
  sort -r | convesc |
  while read -rd '' ; do

    if rmdir -- "$ROOT/$REPLY" ;then
      print_v "$REPLY"

    else
      printf '%s\0' "$REPLY" >> "$_removepkg_bkup_and_remove_dir_list"
    fi

  done 2>/dev/null
}

removepkg_remove_dirs_print_skipped() {
  unconvesc < "$_removepkg_bkup_and_remove_dir_list"
  rm -f "$_removepkg_bkup_and_remove_dir_list" 2>/dev/null
}

removepkg_remove_cats() {
  local REPLY regex="^(.*/)man(./[^/]*$)"
  grep 'man./[^/]*$' | grep -v '/$' | convesc |
  while read -rd '' ; do
    [[ $REPLY =~ $regex ]] &&
    rm -f "${BASH_REMATCH[1]}cat${BASH_REMATCH[2]}"
  done >/dev/null 2>&1
}

removepkg_bkup_restore() {
  [[ -f "$_removepkg_bkup_and_remove_files_bkup" ]] || return
  restore "$_removepkg_bkup_and_remove_files_bkup" "$ROOT"
  removepkg_bkup_rm
} 2>/dev/null

removepkg_bkup_rm() {
    rm -f "$_removepkg_bkup_and_remove_files_bkup"
  _removepkg_bkup_remove_files_bkup=
  _removepkg_bkup_and_remove_files_state=1
}
