#       Copyright 2010 Anshul Narula aka sunny <sunny_delhi18@yahoo.com>
#
#       This file is part of zenpakt

#       Zenpakt is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation; either version 2 of the License, or
#       (at your option) any later version.
#
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#       MA 02110-1301, USA.

$DOC

======================
 removepkg_select_pkg
======================

SYNOPSIS
--------

removepkg_select_pkg var pkg...

DESCRIPTION
-----------

Select among the given packages and store them in var depending on values of
REMOVEALL and ASK.

If REMOVEALL is 1, then select all the packages if they are less than
REMOVEPKG_MAXRM.As a special case, if REMOVEALL is 0, select all the packages.
Otherwise, If REMOVEALL is 2, select the given package which should be only one
in number.This is assumed and not verified because error checking must have
been done while finding the package.In case ASK is 1 and REMOVEALL is 0,
ask the user to confirm removal for each package one by one. Otherwise, fallback
to default behaviour and store the given packages (which should be a single
package, error checking is not done again as it should have been done before
while finding the packages) in var.

See the description for removepkg_find_pkg too.

RETURN
------
0  success
ERR_META_MAXRM  in case the matched packages exceed REMOVEPKG_MAXRM and REMOVEALL is 1

$ENDDOC

$$

removepkg_select_pkg() {
  local _removepkg_select_pkg

  if ((REMOVEALL == 1)) ; then
    ((REMOVEPKG_MAXRM? ($#-1 > REMOVEPKG_MAXRM) :0)) &&
    return "$ERR_META_MAXRM"

    IFS=' ' read -a "$1" < <(
      printf '%q ' "${@:2}"
      printf \\n
    )

  elif ((REMOVEALL == 2)) ; then
    printf -v "$1" '%s' "$2"

  elif ((ASK)) ; then
    for((
      _removepkg_select_pkg=2;
      _removepkg_select_pkg <= $#;
      _removepkg_select_pkg++
    ))
    do
      yesno "remove ${@:_removepkg_select_pkg:1}" &&
      printf -v "$1[$((_removepkg_select_pkg - 2))]" '%s' \
        "${@:_removepkg_select_pkg:1}"

    done
    return 0

  else
   printf -v "$1" '%s' "$2"

  fi
}
