#       Copyright 2011 Anshul Narula aka sunny <sunny_delhi18@yahoo.com>
#
#       This file is part of zenpakt

#       Zenpakt is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation; either version 2 of the License, or
#       (at your option) any later version.
#
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#       MA 02110-1301, USA.

$DOC
=====================
 upgradepkg_find_pkg
=====================

SYNOPSIS
--------

upgradepkg_find_pkg pattern variable

DESCRIPTION
-----------

This is like remove_find_pkg, except it doesnot support / based glob matching
patterns for package patterns as it calls meta_find_pkg_noglob* functions
directly.

Default behaviour is to remove all matched package, ie REMOVEALL is 1 here by
default.All the packages matched are stored( the check for maximum removals
allowed is not done here). If -u|--unique was specified, ie when REMOVEALL is 2,
the package matched must be the only match. .If --ask was specified
(ASK is 1), again all matched packages are stored.


Unlike removepkg, the difference between REMOVALL=1 and ASK=1 is that in case of
REMOVEALL=1 all packages are directly selected for removal only if ASK=0 in case
the number of matched packages is less than REMOVEPKG_MAXRM. In case of
removepkg, if REMOVEALL=1 all packages are selected directly if less than
REMOVEPKG_MAXRM even if ASK=1.

If ASK is 1, all packages matched are asked one by one for users confirmation
instead of falling to default behaviour when REMOVEALL is 1 and ASK is zero.

If REMOVEALL is 2, means the user specifically wants to remove a package only if
a single match was found.In that case we call meta_find_pkg_noglob_unique.
Otherwise, if REMOVEALL is 1(ie we remove all packages matching the given pattern
if number of such packages is within specified limit) or if ASK mode is active we
call to meta_find_pkg_noglob to get a list of all matched package.

If none of above conditions are met, ie REMOVEALL and ASK are zero,(REMOVEALL
being zero is not possible currently),We use the first among the matched
packages.

RETURN
------
* 0 success
* ERR_META_NOTFOUND  no match
* ERR_META_NOTUNIQUE  matched package is not the only match (when REMOVEALL is 2)

(these error statuses are defined in removepkg.functions)

$ENDDOC

$$

upgradepkg_find_pkg() {
  #If the pattern contains a .tgz .tbz .tlz or .txz extension, remove it before
  #passing it to meta_find_package only if if the pattern doesnot start with a /
  if [[ $1 = *.t[glbx]z ]] ; then
    set -- "${1%.*}" "${@:2}"
  fi

  if ((REMOVEALL == 2)) ; then
    meta_find_pkg_noglob_unique "$@" || {
      case $? in
	1)
	  return "$ERR_META_NOTFOUND"
	  ;;
	2)
	  return "$ERR_META_NOTUNIQUE"
	  ;;
      esac
    }

  elif ((REMOVEALL == 1 || ASK)) ; then
    meta_find_pkg_noglob "$@" || return "$ERR_META_NOTFOUND"

  #This case doesnot happen as REMOVEALL is by default is 1 and can be changed
  #to 2 with --unique|-u.I will just keep it around in case I decide to
  #implement the --remove-first or something similar.
  else
    meta_find_pkg_noglob_first "$@" || return "$ERR_META_NOTFOUND"
  fi
}
