#       Copyright 2011 Anshul Narula aka sunny <sunny_delhi18@yahoo.com>
#
#       This file is part of zenpakt

#       Zenpakt is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation; either version 2 of the License, or
#       (at your option) any later version.
#
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#       MA 02110-1301, USA.
$DOC
====================
 upgradepkg_move_db
====================

SYNOPSIS
--------

upgradepkg_move_db_tmp basename...

upgradepkg_move_db_tmp_db var

upgradepkg_move_db_tmp_doinst var

upgradepkg_move_db

upgradepkg_unmove_db

upgradepkg_move_db_tmp_clean

DESCRIPTION
-----------

upgradepkg_move_db_tmp creates two temporary directories under ADM_DIR/setup/tmp
and moves package index and doinst scripts for given basename to these
directories.

upgradepkg_move_db_tmp_db and upgradepkg_move_db_tmp_doinst store the temporary
directory used for storing package index and doinst scripts by
upgradepkg_move_db_tmp respectively in given var name.

upgradepkg_move_db_tmp_state returns the state of a call to upgradepkg_move_db
functions.Initially the state is 1, when upgradepkg_move_db_tmp is called state
is set to 0.If state is zero, you should call upgradepkg_move_db to move the
files to removed_ directories or upgradepkg_unmove_db to move them back to their
original place.These two functions call upgradepkg_move_db_tmp_clean which removes
temporary directories and resets the state.

$ENDDOC
$$

#the temporary directory used to store the database files for the given packages
#before preinstall
_upgradepkg_move_db_tmp_db=

#the temporary directory used to store the doinst scripts for given packages
_upgradepkg_move_db_tmp_doinst=

#the state for database movement
_upgradepkg_move_db_tmp_state=1

upgradepkg_move_db_tmp_db() {
  printf -v "$1" '%s' "$_upgradepkg_move_db_tmp_db"
}

upgradepkg_move_db_tmp_doinst() {
  printf -v "$1" '%s' "$_upgradepkg_move_db_tmp_doinst"
}

upgradepkg_move_db_tmp() {
  local i

  _upgradepkg_move_db_tmp_state=0

  mktemp _upgradepkg_move_db_tmp_db -d "$ADM_DIR/setup/tmp/${RANDOM}XXXXXX" ||
  return "$ERR_CREAT_TMP"

  mktemp _upgradepkg_move_db_tmp_doinst -d "$ADM_DIR/setup/tmp/${RANDOM}XXXXXX" ||
  return "$ERR_CREAT_TMP"

  for i do

    mv -- "$ADM_DIR"/packages/"$i" "$_upgradepkg_move_db_tmp_db" ||
    return "$ERR_MOVE_DB"

    if [[ -f "$ADM_DIR"/scripts/"$i" ]] ; then
      mv -- "$ADM_DIR"/scripts/"$i" "$_upgradepkg_move_db_tmp_doinst" ||
      return "$ERR_MOVE_DB"
    fi

  done
}

upgradepkg_move_db_tmp_state(){
  return "$_upgradepkg_move_db_tmp_state"
}

upgradepkg_move_db_tmp_clean(){
  rm -rf "$_upgradepkg_move_db_tmp_db" "$_upgradepkg_move_db_tmp_doinst"
  _upgradepkg_move_db_tmp_state=1
  _upgradepkg_move_db_tmp_db=
  _upgradepkg_move_db_tmp_doinst=
}

upgradepkg_move_db() {
  local f=()
  trap 'shopt -u nullglob' RETURN
  shopt -s nullglob

  if [[ $_upgradepkg_move_db_tmp_db && -d $_upgradepkg_move_db_tmp_db ]] ; then
    f=("$_upgradepkg_move_db_tmp_db"/*)

    #The package database must be there in case we are upgrading
    ((${#f[@]})) || return "$ERR_MOVE_DB"

    mv -- "${f[@]}" "$ADM_DIR"/removed_packages/ ||
    return "$ERR_MOVE_DB"
  fi

  if [[ $_upgradepkg_move_db_tmp_doinst && -d $_upgradepkg_move_db_tmp_doinst ]] ; then
    f=("$_upgradepkg_move_db_tmp_doinst"/*)

    ((${#f[@]})) && {
      mv -- "${f[@]}" "$ADM_DIR"/removed_scripts/ ||
      return "$ERR_MOVE_DB"
    }
  fi

  upgradepkg_move_db_tmp_clean
}

upgradepkg_unmove_db() {
  local f
  shopt -s nullglob

  if [[ $_upgradepkg_move_db_tmp_db && -d $_upgradepkg_move_db_tmp_db ]] ; then
    f=("$_upgradepkg_move_db_tmp_db"/*)

    ((${#f[@]})) && mv -- "${f[@]}" "$ADM_DIR"/packages/
  fi

  if [[ $_upgradepkg_move_db_tmp_doinst && -d $_upgradepkg_move_db_tmp_doinst ]] ; then
    f=("$_upgradepkg_move_db_tmp_doinst"/*)

    ((${#f[@]})) && mv -- "${f[@]}" "$ADM_DIR"/scripts/
  fi
  shopt -u nullglob
  upgradepkg_move_db_tmp_clean
}
