#       Copyright 2010 Anshul Narula aka sunny <sunny_delhi18@yahoo.com>
#
#       This file is part of zenpakt

#       Zenpakt is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation; either version 2 of the License, or
#       (at your option) any later version.
#
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#       MA 02110-1301, USA.

$$

upgradepkg_parseargs() {

  local getargs removeall

  #REMOVEALL has three values it can be only 1, ie in case of --warn
  #the value 0 which corresponds to --remove-first is not implemented yet
  removeall=(
    [0]="--remove-first|-F"
    [1]="--warn|-w"
    [2]="--unique|-u"
  )

  getargs=(
    w    warn            -  1
    d    dry-run         -  1
    a    ask             -  1
    R    raw             -  1
    V    no-verbose      -  1
    r    root            :  1
    m    max             :  1
    u    unique          -  1
    s    separator       :  1
    I    reinstall       -  1
    i    install-new     -  1
  )

  _upgradepkg_parseargs_set_mode_() {
    if ! readonly MODE=$1 2>/dev/null ; then
      error_print "mutually exclusive modes: $MODE and $1"
      return 1
    fi
  }

  _upgradepkg_parseargs_set_removeall_() {
    if ! readonly REMOVEALL=$1 2>/dev/null ; then
      error_print \
        "incompatible options: $2 and ${removeall[REMOVEALL]} cannot be used together"
      return 1
    fi
  }
  upgradepkg_parseargs() {

    case "$2" in

      dry-run|d|warn|w)
        _upgradepkg_parseargs_set_mode_ warn || return 1
        _upgradepkg_parseargs_set_removeall_ 1 "$1$2" || return 1
        RAW=1 VERBOSE=1
        ;;

      install-new|i)
        UPGRADEPKG_INSTALLNEW=1
        ;;

      reinstall|I)
        UPGRADEPKG_REINSTALL=1
        ;;

      no-verbose|V)
        _upgradepkg_parseargs_set_mode_ quiet || return 1
        VERBOSE=0 RAW=1
        ;;

      ask|a)
        _upgradepkg_parseargs_set_mode_ ask || return 1
        ASK=1 RAW=0
        ;;

      raw|R)
        _upgradepkg_parseargs_set_mode_ raw || return 1
        RAW=1
        ;;

      unique|u)
        _upgradepkg_parseargs_set_removeall_ 2 "$1$2" || return 1
        ;;

      max|m)
        if [[ $3 != [0-9]*([0-9]) ]] ; then
          error_print "$3 is not a valid value for $1$2"
          return 1
        fi
        REMOVEPKG_MAXRM=$3
        ;;

      fast|f)
        PARANOID=0
        ;;

      separator|s)
        if [[ $3 = *@(/|[a-zA-Z0-9]|-|.)* ]] || ((${#3} == 0 || ${#3} > 3));then
          error_print "%s: invalid separator: $3"
          return 1
        fi
        UPGRADEPKG_FS=${ARG_OPTARG[count]}
        ;;

      root|r)
        if [[ ! -d $3 ]] ; then
          error_print "root directory $3 does not exist"
          return 1
        fi
        fullpath "$3" ROOT
        ;;

      version)
        version 'GPL2+'
        exit 0
        ;;

      help)
        usage 0 upgradepkg
        exit 0
        ;;

    esac

  }

  trap 'unset -f upgradepkg_parseargs _upgradepkg_parseargs_set_removeall_ \
    _upgradepkg_parseargs_set_mode_' RETURN

  if ! getargs upgradepkg_parseargs "$@" || (($GETARGS_IND > $#)) ; then
    VERBOSE=1 ASK=0 RAW=1
    usage 1
    return 1
  fi

  readonly MODE RAW REMOVEPKG_MAXRM ASK ROOT REMOVEALL PARANOID VERBOSE \
    UPGRADEPKG_INSTALLNEW UPGRADEPKG_REINSTALL UPGRADEPKG_FS || return 0
}
