#       Copyright 2011 Anshul Narula aka sunny <sunny_delhi18@yahoo.com>
#
#       This file is part of zenpakt

#       Zenpakt is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation; either version 2 of the License, or
#       (at your option) any later version.
#
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#       MA 02110-1301, USA.

$DOC

=======================
 upgradepkg_select_pkg
=======================

SYNOPSIS
--------

upgradepkg_select_pkg var pkg_to_install pkg_to_remove...

DESCRIPTION
-----------

Select among the given packages to be removed and store them in var depending
on values of REMOVEALL and ASK.

If ASK is 1, then ask the user to confirm for removal of each found package.
Otherwise, If REMOVEALL is 2, select the given package which should be only one
in number.This is assumed and not verified because error checking must have
been done while finding the package. If REMOVEALL is 1 (the default), then
select all the packages if they are less than REMOVEPKG_MAXRM.As a special case,
if REMOVEALL is 0 (the default), select all the packages.

(This never happens:
Otherwise, store the given packages (which should be a single
package, error checking is not done again as it should have been done before
while finding the packages) in var.).

See the description for upgradepkg_find_pkg too.

RETURN
------
0  success
ERR_META_MAXRM  in case the matched packages exceed REMOVEPKG_MAXRM and REMOVEALL is 1

$ENDDOC

$$

upgradepkg_select_pkg() {
  local _upgradepkg_select_pkg_reply _upgradepkg_select_pkg

  if ((ASK)) ; then
    print_v "found $(($#-2)) package(s) to be upgraded with $2"

    for((
      _upgradepkg_select_pkg=3;
      _upgradepkg_select_pkg <= $#;
      _upgradepkg_select_pkg++
    ))
    do
      yesno "remove ${@:_upgradepkg_select_pkg:1}" &&
      printf -v "$1[$((_upgradepkg_select_pkg - 3))]" '%s' \
        "${@:_upgradepkg_select_pkg:1}"
    done

  elif ((REMOVEALL == 2)) ; then
    printf -v "$1" '%s' "$2"

  elif ((REMOVEALL == 1)) ; then
    ((REMOVEPKG_MAXRM? ($# -2 > REMOVEPKG_MAXRM) :0)) &&
    return "$ERR_META_MAXRM"

    IFS=' ' read -a "$1" < <(
      printf '%q ' "${@:3}"
      printf \\n
    )

  else
   printf -v "$1" '%s' "$2"

  fi
}
