$INCLUDE ./config.bash
$$

#       Copyright 2010 Anshul Narula aka sunny <sunny_delhi18@yahoo.com>
#
#       This file is part of zenpakt

#       Zenpakt is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation; either version 2 of the License, or
#       (at your option) any later version.
#
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#       MA 02110-1301, USA.

#========
# SOURCE
#========
#source required .functions files
. "$ZENPAKT_INCLUDE_DIR"/common explodepkg || {
  printf "%s\n" "failed to source function" >&2
  exit 1
}

#source configfile here using sourceconf() in common.functions, for next zenpakt
#version

#=========================
# CONFIGURATION VARIABLES
#=========================

ZENPAKT_PROG_VERSSTR="explodepkg (zenpakt)"

# verbose is set to 1 ; this means all messages and errors output to screen
readonly VERBOSE=1

if ask_init ; then
  readonly RAW=0
  col_fg green red cyan
  txt_attrib bold reset
else
  readonly RAW=1
fi

#=================
# PARSE ARGUMENTS
#=================
#We dont accept any arguments except version and help;)
if [[ $1 = -* ]] ; then
  case $1 in
    --|-)
      shift
      ;;

    --version)
      version 'GPL2+'
      exit 0
      ;;

    --help)
      usage 0 explodepkg
      exit 0
      ;;

    *)
      error_print "inavlid option: $1"
      usage 1
      error_exit "$ERR_OPT"
      ;;
  esac
fi

tar || error_exit "$ERR_NO_TAR"

spin_msg "verifying $1..."

#check wether the package is a file or url
pkg_is_file_or_url "$1" REPLY ||
error_exit "$ERR_NOTEXIST"

[[ $REPLY != file ]] && error_exit "$ERR_NOTEXIST"

#get various data obtained from name of package
pkg_parse_name "$1" _ _ compression _ ||
error_exit "$ERR_INVAL_EXT"

spin_msg_success

spin_msg "checking for $compression..."

# if external compression utility doesn't exist
chk_cmd "$compression" || error_exit "$ERR_NO_COMPRESSUTIL"

spin_msg_success

print_v "Exploding $1:"
explodepkg_explode "$compression" < "$1" || error_exit "$ERR_INST_EXTRACT"

if [[ -f install/doinst.sh ]] ; then
  warn_v "An installation script was detected in ./install/doinst.sh, but was not executed."
fi

exit 0
