$INCLUDE ./config.bash

==========
 PROBLEMS
==========

These are some problems that have to be solved in future, mainly due to
compatibility with earlier slackware pkgtools and their package format.

 * symbolic links with newlines, hopefully would not happen

    I basically cannot understand why they didnt they add a list of links
    inside the package instead of or in addition to adding them to a
    doinst script.At least create a dolist script along
    with it; Whatever extracting links from a doinst script during
    removal is just not that simple.

 * The ( cd dir ; rm .. fromat of adding symbolic links to a doinst.sh

   First it should be cd dir && instead of a ;
   Second the dir and others should have been quoted too.

 * State of the file system may change by the time we create the tar ball

   which may result in , .eg., you may have another symbolic link that was never
   added to the doinst.sh.Use a temporary file and consider the modification time
   for a file before adding it to the tar package

 * Asking for adding or not adding symbolic links just complicates the entire

   process; so we just default to not adding it and give a warning instead of
   actually asking.I prefer to lose compatibility over here at least.

$$

#       Copyright 2010 Anshul Narula aka sunny <sunny_delhi18@yahoo.com>
#
#       This file is part of zenpakt

#       Zenpakt is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation; either version 2 of the License, or
#       (at your option) any later version.
#
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#       MA 02110-1301, USA.



#========
# SOURCE
#========
#source required .functions files
. "$ZENPAKT_INCLUDE_DIR/"common makepkg || {
  printf '%s\n' "failed to source function" >&2
  exit 1
}

#source configfile here using sourceconf() in common.functions, for next zenpakt
#version

#=========================
# CONFIGURATION VARIABLES
#=========================

#The canonical name
ZENPAKT_PROG_CANON_NAME=makepkg

#The version string
ZENPAKT_PROG_VERSSTR="makepkg (zenpakt)"

#PARANOID=1

# verbose is set to 1 ; this means all messages and errors output to screen
readonly VERBOSE=1

if ask_init ; then
  readonly RAW=0
  col_fg green red cyan
  txt_attrib bold reset
else
  readonly RAW=1
fi

MAKEPKG_PREPEND=0
MAKEPKG_LINKADD=1
MAKEPKG_CHOWN=

#exit on SIGINT or SIGTERM
#Using error_print terminates an active spin too
trap 'error_print "received SIGINT, exiting..." ; exit' INT
trap 'error_print "received SIGTERM, exiting..." ; exit' TERM

#Set proper umask
umask 022

#check tar exists and if so which version
tar || error_exit "$ERR_NO_TAR"

#parse arguments and if it fails we exit
makepkg_parseargs "$@" || error_exit "$ERR_OPT"
shift $((GETARGS_IND-1))

if ((MAKEPKG_PREPEND)) ; then
  #cleanup on exit
  trap '((BASHPID == $$)) && makepkg_cleanup' EXIT
  #Like meta_init_tmpdir
  #create a temporary directory under TMPDIR
  mktemp_init_tmpdir "${TMPDIR:-/tmp}" || error_exit "$ERR_CREAT_TMP"
fi

#=====================
#  PARSE PACKAGE NAME
#=====================
#get various data obtained from name of package
#ensure full path for $package, as we will be cd'ing later
spin_msg "verifying $1..."

unset package basename compression name version arch release build extension

pkg_parse_name "$1" package basename compression name version arch release \
  build extension || error_exit "$ERR_INVAL_EXT"

[[ ${package%/*} = @("$PWD"|"$PWD"/*) ]] && error_exit "$ERR_IN_PWD"

readonly package basename compression name version arch release build extension

spin_msg_success

printf_v '%spackage name:%s %s\n' "$TXT_bold" "$TXT_reset" "$name"
printf_v '%spackage version:%s %s\n' "$TXT_bold" "$TXT_reset" "$version"
printf_v '%spackage arch:%s %s\n' "$TXT_bold" "$TXT_reset" "$arch"
printf_v '%spackage release:%s %s\n' "$TXT_bold" "$TXT_reset" "$release"
printf_v '%spackage build:%s %s\n' "$TXT_bold" "$TXT_reset" "$build"
printf_v '%spackage extension:%s %s\n' "$TXT_bold" "$TXT_reset" "$extension"
printf_v '%spackage compression:%s %s\n' "$TXT_bold" "$TXT_reset" "$compression"

spin_msg "checking for $compression..."

chk_cmd "$compression" || error_exit "$ERR_NO_COMPRESSUTIL"

spin_msg_success

#=================
# create install/
#=================
if [[ ! -d install/ ]] ; then
  warn_v "no install/ directory found"
  spin_msg "creating install/..."
  rm -f install/
  mkdir install/ || error_exit "$ERR_MKDIR_INSTALL"
  spin_msg_success
fi

#=============
# PERMISSIONS
#=============
##changing permissions
#makepkg_chown
print_v "This next step is optional-you can set the permissions of all" \
  "directories in your package to 755 (drwxr-xr-x) and directory ownerships " \
  "to root.root.If any of the directories in your package have" \
  "special permissions, then DO NOT reset them here!" ""

if [[ $MAKEPKG_CHOWN ]] ; then
  echon_v "reset all directory permissions and ownerships? ([y]es/[n]o)"
  if ((MAKEPKG_CHOWN)) ; then
    print "yes"
  else
    print "no"
  fi
elif yesno "reset all directory permissions and ownerships" ; then
    readonly MAKEPKG_CHOWN=1
else
  readonly MAKEPKG_CHOWN=0
fi

#No error checking, though we do not hide the errors and output from chown
if ((MAKEPKG_CHOWN)) ; then
  makepkg_chown
fi

#================
# EXTRA WARNINGS
#================

#warnings
# Warn of zero-length files,
# find printf shall handle weird characters if output goes to terminal
find . -type f -size 0c -printf "$ZENPAKT_PROG_NAME: WARNING:  zero length file %P\n" >&2

#Warn of corrupt or empty gzip files
#find's printf shall handle weird characters if output goes to terminal
find . -type f -name '*.gz' >&2 2>/dev/null \( \
  -exec gzip -t {} \; -o -printf "$ZENPAKT_PROG_NAME: WARNING:  gzip test failed on %P\n" \
\) -o -type f -name '*.gz' \( \
  -exec "$BASH" -c '[[ $(gzip -dc "$1" | wc -c) = 0 ]]' _ {} \; \
  -printf "$ZENPAKT_PROG_NAME: WARNING: %P is an empty gzipped file\n" \
\)

#Some more handy warnings, which are given by original pkgtools:
if [[ -d usr/share/man ]]; then
  warn_v "/usr/share/man (with possibly not gzipped man pages) detected"
fi

if [[ -d usr/share/info ]]; then
  warn_v "/usr/share/info (with possibly not gzipped info pages) detected"
fi

find . -type d -name site_perl  >&2 \
  -printf "$ZENPAKT_PROG_NAME: WARNING: site_perl directory detected (this is fine for a local package build)\n"

#============
# SLACK-DESC
#============

if [[ ! -s install/slack-desc ]] ; then

  rm -rf install/slack-desc
  warn_v "no or zero length slack-desc found,using default slack-desc instead"
  printf '%s\n' > install/slack-desc \
    "$name: $name $version" \
    "$name:" \
    "$name: Generated by Zenpakt $ZENPAKT_PROG_CANON_NAME on $(date)" \
    "$name:" \
    "$name:" \
    "$name:" \
    "$name:" \
    "$name:" \
    "$name:" \
    "$name:" \
    "$name:"
fi


#check the slack-desc
if grep -v "^$name:" install/slack-desc | grep -qv "^$basename:" ; then
  warn_v "slack-desc contains lines that do not start with the package name"
fi

#check the slack-desc line length
if [[ $(wc -l < install/slack-desc) != 11 ]] ; then
  warn_v "Slack-desc doesnot contain 11 lines"
fi

#check the column length must be 72, thats what they use in the handy ruler
#anyway

#================
# SYMBOLIC LINKS
#================
if ((MAKEPKG_LINKADD)) ; then
  makepkg_link_add || error_exit $?
elif makepkg_link_exists ; then
  warn_v "Found symbolic links but not added to doinst.sh"
fi

#================
# CREATE PACKAGE
#================

spin_msg "creating package..."

rm -f "$package"
makepkg_make "$compression" > "$package" 2>/dev/null || {
  error_print "$compression returned with exit status $?"
  error_exit "$ERR_CREAT_PKG"
}

spin_msg_success

#======
# EXIT
#======

#No need to use error_exit here because we dont use error_continue
#If we reached here we need to exit with zero exit status
exit 0
