$INCLUDE ./config.bash
$$

#       Copyright 2010 Anshul Narula aka sunny <sunny_delhi18@yahoo.com>
#
#       This file is part of zenpakt

#       Zenpakt is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation; either version 2 of the License, or
#       (at your option) any later version.
#
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#       MA 02110-1301, USA.

#========
# SOURCE
#========
#source required .functions files
#Upgradepkg.functions sources removepkg and installpkg.functions
. "$ZENPAKT_INCLUDE_DIR"/common upgradepkg || {
  printf '%s\n' "failed to source function" >&2
  exit 1
}

#source configfile here using sourceconf() in common.functions, for next
#zenpakt version

#=========================
# CONFIGURATION VARIABLES
#=========================

ZENPAKT_PROG_VERSSTR="upgradepkg (zenpakt)"

# extra checks/backup while upgrading packages
PARANOID=1

MD5SUM=0

# default ROOT is / ;specify another as an env var or use -r|--root
# env var is silently replaced with / if the directory specified by it
# doesnot exist, while an error occurs in case of -r|--root
if [[ -d $ROOT ]] ; then
  fullpath "$ROOT" ROOT
else
  ROOT=/
fi

# default MODE ; <do not change>
MODE=upgrade

# verbose is set to 1 ; this means all messages and errors output to screen
VERBOSE=1

# non interactive mode by default
ASK=0

# no-spin or raw mode is off by default
RAW=0

#quit if more than this number of packages are found to be matching the desired
#pkgname given in removeall mode
# default is 0 which means infinity
REMOVEPKG_MAXRM=0

#default behaviour like the original upgradepkg is to remove all matches if number
#of packages is less than REMOVEPKG_MAXRM.If set to 0, remove one among many
#matches.If set to 2, remove only packages if there is a unique match for
#specified name
REMOVEALL=1

#% though uncommon may be a part of package names and therefore you may specify
#a different separator.Alphanumeric characters, / , - , . are strictly not allowed
#Separator can be more than one character long but not more than 3.Be sure to
#use a separator that is not present in any of package names given on command
#line.The first occurence of the separator is treated as the division point to
#break the packages into names
UPGRADEPKG_FS=%

#installing new packages is not done by default
UPGRADEPKG_INSTALLNEW=0

#reinstall is not done by default
UPGRADEPKG_REINSTALL=0

upgradepkg_main() {

  local package package_install package_remove basename compression name REPLY \
    found=() selected=() status compressed uncompressed full_desc desc md5sum

  for package do

    #initial checks that prevent further processing in case a necessary
    #condition for a package is not specified; this introduces a race condition
    #There should always be a double check for possible failures during
    #upgrade

    spin_msg "parsing $package..."

    #getting the probable package names to be upgraded
    upgradepkg_divide_name "$package" package_install package_remove ||
    error_continue $? "$package"

    spin_msg_success

    spin_msg "verifying $package_install..."


    #check wether the package to be installed is a file or url
    pkg_is_file_or_url "$package_install" REPLY ||
    error_continue "$ERR_NOTEXIST" "$package_install"

    [[ $REPLY != file ]] && error_continue "$ERR_NOTEXIST"  "$package_install"


    #get various data obtained from name of package
    #ensure full path for $package_install, as we will be cd'ing later
    pkg_parse_name "$package_install" package_install basename compression name \
    || error_continue "$ERR_INVAL_EXT" "$package_install"

    spin_msg_success

    #first we check if the package is already installed, we will do a search
    #with the $name later.
    [[ -f $ADM_DIR/packages/$basename && $UPGRADEPKG_REINSTALL = 0 ]] &&
    error_continue "$ERR_INSTALLED" "$package_install"


    #If no package basename was specified for removal use the one for
    #package to be installed
    [[ $package_remove ]] || package_remove=$name

    spin_msg "searching for '$package_remove'..."


    #check for existence of $package_remove and guess installed name
    found=() selected=()

    if upgradepkg_find_pkg "$package_remove" found ; then

      spin_msg_success

      #select among found packages
      upgradepkg_select_pkg selected "$basename" "${found[@]}" ||
      error_continue $? "$package_remove"

      #selected may be empty in case of ask mode
      ((${#selected[@]})) || error_continue "$ERR_META_NOTSELECTED" "$package_remove"

    else
      status=$?
      ((UPGRADEPKG_INSTALLNEW)) || error_continue $status "$package_remove"

      spin_msg_error

    fi

    #acting on packages to be removed according to the mode
    case "$MODE" in

      warn)
        if ((${#selected[@]})) ; then
          printf '>>>%s' "$package_install will be upgraded with:"
          printf \'%s\' "${selected[@]}"
          printf \\n
        else
          printf '>>>%s\n' "$package_install will be newly installed"
        fi
        ;;

      upgrade|ask|raw|quiet)

        spin_msg "looking for package description..."

        pkg_desc "$package_install" "$compression" "$basename" "$name" compressed \
          uncompressed full_desc desc md5sum

        spin_msg_success

        print_v "" "${TXT_bold}PACKAGE: ${TXT_reset}$basename" \
          "${TXT_bold}PACKAGE DESCRIPTION:${TXT_reset}"
        echon_v "$desc"
        print_v "${TXT_bold}SIZE:${TXT_reset}" \
          "Compressed: $compressed" \
          "Uncompressed: $uncompressed" ""

        if ((${#selected[@]}));then

          if ((ASK)) ; then
            print_v "upgrading $basename will remove ${#selected[@]} package(s)"
            yesno "upgrade $basename" ||
            error_continue "$ERR_META_NOTSELECTED" "$package_install"
          fi

          #first set the state for the upgrade
          #If we are interrupted by a signal we have to undo changes
          upgradepkg_state_start upgrade "$basename"

          #Make sure an error_continue changes the state back for next loop
          error_callback continue 'upgradepkg_state_end;error_callback continue'

          #Before preinstalling the package we move all the database files and
          #doinst files to temporary directory, we dont want it to be overwritten
          upgradepkg_move_db_tmp "${selected[@]}" || {
            status=$?
            upgradepkg_unmove_db
            error_continue $status "$package_remove"
          }

          #Unlike in installpkg we do the backup after setting the state.
          #This is necessary because we need to move the database files for
          #packages to be removed to a temporary directory first.Backup will also
          #remove database file if we are reinstalling which is not what we want
          if ((PARANOID)) ; then
            spin_msg "backing up files..."

            installpkg_bkup "$package_install" "$compression" "$basename" || {
              status=$?
              upgradepkg_unmove_db
              error_continue $status "$package_install"
            }

            spin_msg_success
          fi


          spin_msg "preinstalling $basename..."

          #next we install the given package
          installpkg_install "$package_install" "$compression" "$basename" \
            "$compressed" "$uncompressed" full_desc "$md5sum" || {
            status=$?
            installpkg_move_db_partial "$basename"
            installpkg_bkup_exists && installpkg_bkup_restore
            upgradepkg_unmove_db
            upgradepkg_state_end
            error_continue $status "$package_install"
          }

          spin_msg_success

          spin_msg "configuring $basename..."

          #We continue the upgrade even if configure fails.Just record the error
          #for the summary later.0 argument to error_continue not actually
          #call continue
          installpkg_configure "$package_install" "$compression" "$basename" ||
          error_continue $? "$package_install" 0

          spin_msg_success

          #next we need to get file and links list for all the packages in
          #actual database and files in temporary directory we created
          #and find unique files to be removed

          spin_msg "fetching file list..."

          mktemp list
          upgradepkg_file_list_unique > "$list" || {
            status=$?
                  mv "$ADM_DIR/"{,partial_}packages/"$basename"
            rm -f "$ADM_DIR/scripts/$basename"
            installpkg_bkup_exists && installpkg_bkup_restore
            upgradepkg_unmove_db
            upgradepkg_state_end
            error_continue $status "$package_remove"
          }

          spin_msg_success

          #Now we actually remove the files
          spin_msg "removing files..."

          #If we fail here we need to not only restore from our backup but
          #also remove the installed package and then restore from the backup
          #created for the package overwrite
          #Or a better way which would simplify the course of action for
          #other tools:
          #Create another index under ADM_DIR where we move the index
          #of the newly installed package and just restore from our backups
          #Then leave it for users to use another tool to clean the backup or
          #reinstall the package.For this we would require changes in
          #installpkg_cleanup also and a new tool to check for cleanup or
          #reinstalling the partially downloaded files--pkgdb
          #options :  status, clean, realclean, remove-partial,reinstall-partial

          removepkg_remove_files >/dev/null < "$list" || {
            status=$?
            removepkg_bkup_restore
            mv "$ADM_DIR/"{,partial_}packages/"$basename"
            rm -f "$ADM_DIR/scripts/$basename"
            installpkg_bkup_exists && installpkg_bkup_restore
            upgradepkg_unmove_db
            upgradepkg_state_end
            error_continue $status "$package_remove"
          }

          spin_msg_success

          removepkg_remove_cats < "$list"

          spin_msg "removing empty directories..."

          removepkg_remove_dirs >/dev/null < "$list"

          spin_msg_success

          #If we reach here we have already done the first install and we are
          #doing it second time to ensure that some files are not missing
          #after the removal.As such, we are either overwriting files from our
          #preinstall step or are just placing back missing files.So a backup
          #of overwritten files is not done now.

          spin_msg "installing $basename..."

          installpkg_install "$package_install" "$compression" "$basename" \
          "$compressed" "$uncompressed" full_desc "$md5sum" || {
            status=$?
            removepkg_bkup_restore
            mv "$ADM_DIR/"{,partial_}packages/"$basename"
            rm -f "$ADM_DIR/scripts/$basename"
            installpkg_bkup_exists && installpkg_bkup_restore
            upgradepkg_unmove_db
            upgradepkg_state_end
            error_continue $status "$package_install"
          }

          spin_msg_success

          spin_msg "reconfiguring $basename..."

          installpkg_configure "$package_install" "$compression" "$basename" ||
          error_continue $? "$package_install" 0

          spin_msg_success

          #move the package database to the removed_ directories
          #where it should be after a removal
          upgradepkg_move_db || {
            status=$?
            removepkg_bkup_restore
            mv "$ADM_DIR/"{,partial_}packages/"$basename"
            rm -f "$ADM_DIR/scripts/$basename"
            installpkg_bkup_exists && installpkg_bkup_restore
            upgradepkg_unmove_db
            upgradepkg_state_end
            error_continue $status "$package_remove"
          }

          #remove a partial database entry if it is there
          rm -f "$ADM_DIR/partial_packages/$basename"

          upgradepkg_state_end

          #remove the backups; they are not needed any more
          installpkg_bkup_rm
          removepkg_bkup_rm

        else
          #Ask one final time in case ASK != 0
          if ((ASK)) ; then
            yesno "install new package $basename" ||
            error_continue "$ERR_META_NOTSELECTED" "$package_install"
          fi

          if ((PARANOID)) ; then
            spin_msg "backing up files..."

            installpkg_bkup "$package_install" "$compression" "$basename" ||
            error_continue $? "$package_install"

            spin_msg_success
          fi

          spin_msg "installing $basename..."

          upgradepkg_state_start install "$basename"

          if installpkg_install "$package_install" "$compression" "$basename" \
            "$compressed" "$uncompressed" full_desc "$md5sum" ; then
            installedpkglist+=("$basename")
            #remove any partial database entry
            rm -f "$ADM_DIR/partial_packages/$basename"
            spin_msg_success

          else
            status=$?
            installpkg_move_db_partial "$basename"
            ((PARANOID)) && installpkg_bkup_restore
            upgradepkg_state_end
            error_continue $status "$package_install"
          fi

          spin_msg "configuring $basename..."

          installpkg_configure "$package_install" "$compression" "$basename" || {
            status=$?
            ((PARANOID)) && installpkg_bkup_rm
            upgradepkg_state_end
            error_continue $status "$package_install"
          }

          spin_msg_success

          #Remove the backup now; we wait till now to remove the backup so
          #we can restore if we receive INT or TERM
          ((PARANOID)) && installpkg_bkup_rm

          upgradepkg_state_end
        fi
        ;;

    esac
  done

  #summary of error messages is not displayed for warn mode
  [[ $MODE = warn ]] && return 0

  error_summary

  if ((ASK)) ; then
    read -n 1 -p "${TXT_bold}press any key to continue${TXT_reset}"
  fi
}

#exit on SIGINT or SIGTERM
#Using error_print terminates an active spin too
trap 'error_print "received SIGINT,exiting..." ; exit' INT
trap 'error_print "received SIGTERM,exiting..." ; exit' TERM

#check wether the user is root or not , this is just a check for code clarity
((UID)) && error_exit "$ERR_NOTROOT"

#Set proper umask
umask 022

#parse arguments and if it fails we exit
upgradepkg_parseargs "$@" || error_exit "$ERR_OPT"

#check tar exists and if so which version
tar || error_exit "$ERR_NO_TAR"

#check for comm
#enable_builtin comm0 || error -v 9


# check if package meta database in given root exists or not
# if install new was not asked and package database doesn't exist we
# have no package to upgrade
if ((!INSTALLNEW)) ; then
  [[ -d $ROOT/var/log/packages ]] || error_exit "$ERR_NO_DB"
fi

# Do some clean up on exit
trap '((BASHPID == $$)) && upgradepkg_cleanup' EXIT

# check lock or create a lock to prevent parallel run
lock || error_exit "$ERR_LOCKED"

# initialize tmpdir, package database should already be there, at least
# partially in most cases.
meta_init_db || error_exit "$ERR_DBINIT"
meta_init_tmpdir || error_exit "$ERR_CREAT_TMP"

#Get some colors and text attributes needed in case both
#RAW is zero and VERBOSE is one
if ((RAW == 0 && VERBOSE == 1)) ; then
  col_fg green red cyan
  txt_attrib bold reset
fi

if ((ASK)) ; then
  ask_init || error_exit $?
fi

#main loop function
upgradepkg_main "${@:GETARGS_IND}"

#exit with the last set exitstatus
error_exit
